/*
 * @(#)PolyLineHandle.java 5.1
 *
 */

package CH.ifa.draw.figures;

import java.awt.*;
import java.awt.event.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.util.Geom;
import CH.ifa.draw.standard.LocatorHandle;

/**
 * A handle for a node on the polyline.
 */
public class PolyLineHandle extends LocatorHandle {

    private int fIndex;
    private Point fAnchor;

   /**
    * Constructs a poly line handle.
    * @param owner the owning polyline figure.
    * @l the locator
    * @index the index of the node associated with this handle.
    */
    public PolyLineHandle(PolyLineFigure owner, Locator l, int index) {
        super(owner, l);
        fIndex = index;
    }

    public void invokeStart(int  x, int  y, DrawingView view) {
        fAnchor = new Point(x, y);
    }

    public void invokeStep (MouseEvent e, int x, int y, int anchorX, int anchorY, DrawingView view) {
	Point p;
	PolyLineFigure plf=myOwner();
	if (e.isControlDown() && fIndex>0 && fIndex<plf.pointCount()-1) {
	  Point prev=plf.pointAt(fIndex-1),
	        next=plf.pointAt(fIndex+1);
	  Point corner1=new Point(prev.x,next.y),
	      corner2=new Point(next.x,prev.y);
	  if (Geom.length(x,y,corner1.x,corner1.y) <
	      Geom.length(x,y,corner2.x,corner2.y))
	      p=corner1;
	  else
	      p=corner2;
	} else {
	  p=new Point(x,y);
	}
        myOwner().setPointAt(p, fIndex);
    }

    public void invokeStart(MouseEvent e, int x, int y, DrawingView view) {
        if (e.getClickCount() > 1) {
	    myOwner().removePointAt(fIndex);
	    view.selectionInvalidateHandles();
        } else {
           invokeStart(x,y,view);
        }
    }

    protected int getIndex() {
	return fIndex;
    }

    protected PolyLineFigure myOwner() {
        return (PolyLineFigure)owner();
    }
}


