/*
 * @(#)ChildDragTracker.java 5.1
 *
 */

package CH.ifa.draw.standard;

import java.awt.*;
import java.awt.event.MouseEvent;
import java.util.*;
import CH.ifa.draw.framework.*;

/**
 * ChildDragTracker implements the dragging of a clicked
 * ChildFigure.
 *
 * @see SelectionTool
 */
public class ChildDragTracker extends DragTracker {

    private boolean doTracking=false;
    private Point startPoint;
    private Figure lastFigure=null;
    private ParentFigure oldParent, newParent=null;

    public ChildDragTracker(DrawingEditor editor, ChildFigure anchor) {
        super(editor,anchor);
        startPoint = anchor.center();
        oldParent=anchor.parent();
    }

    public void mouseDown(MouseEvent e, int x, int y) {
       super.mouseDown(e,x,y);
       doTracking = fAnchorFigure != null && fAnchorFigure instanceof ChildFigure
            && view().selectionCount()==1;
       if (doTracking) {
	   fEditor.drawing().bringToFront(fAnchorFigure);
       }
    }

    protected void drag(int dx, int dy) {
       if (doTracking) {
         fAnchorFigure.moveBy(dx,dy);
       } else {
          super.drag(dx,dy);
       }
    }


    private void findNewParent(int x, int y) {
       Figure figure = drawing().findFigureWithout(x,y,fAnchorFigure);
       if (figure != lastFigure) {
           lastFigure=figure;
           if (figure instanceof ParentFigure
	       && ((ChildFigure)fAnchorFigure).parent() != figure
	       && ((ChildFigure)fAnchorFigure).canBeParent((ParentFigure)figure)) {
                //Toolkit.getDefaultToolkit().beep();
                newParent=(ParentFigure)figure;
                view().clearSelection();
                view().addToSelection(newParent);
            } else if (newParent != null && oldParent != null) {
                view().clearSelection();
                view().addToSelection(oldParent);
                newParent=null;
            }
       }
    }

    public void mouseDrag(MouseEvent e, int x, int y) {
        super.mouseDrag(e, x, y);
        if (doTracking) {
           findNewParent(x,y);
        }
    }

    public void mouseUp(MouseEvent e, int x, int y) {
        super.mouseUp(e, x, y);
        if (doTracking && fAnchorFigure!=null) {
           Point endPoint = fAnchorFigure.center();
           if (!startPoint.equals(endPoint)) {
              findNewParent(x,y);
              if (newParent != null) {
                //System.out.println("New Parent "+newParent+"!");
                fAnchorFigure.moveBy(startPoint.x-endPoint.x,startPoint.y-endPoint.y);
                ((ChildFigure)fAnchorFigure).setParent(newParent);
              }
           }
        }
    }

}
