package CH.ifa.draw.util;

import java.awt.*;
import java.awt.image.*;

public class ContextGraphics extends Graphics {

  protected final Graphics g;
  private final Color background;

  public ContextGraphics(Graphics g, Color background) {
      this.g=g;
      this.background=background;
  }

  public Graphics create() {
      return new ContextGraphics(g.create(),background);
  }

  public Graphics create(int x,
                         int y,
                         int width,
                         int height) {
      return new ContextGraphics(g.create(x,y,width,height),background);
  }

  public void dispose() {
      g.dispose();
  }

  public Color getColor() {
      Color color=g.getColor();
      if (color.equals(background)) {
	  return ColorMap.BACKGROUND;
      } else {
	  return color;
      }
  }

  public void setColor(Color c) {
      if (ColorMap.isBackground(c)) {
	  g.setColor(background);
      } else {
	  g.setColor(c);
      }
  }

  public void setPaintMode() {
      g.setPaintMode();
  }

  public void setXORMode(Color otherColor) {
      g.setXORMode(otherColor);
  }

  public Font getFont() {
      return g.getFont();
  }


  public void setFont(Font font) {
    g.setFont(font);
  }

  public FontMetrics getFontMetrics() {
    return g.getFontMetrics();
  }

  public FontMetrics getFontMetrics(Font font) {
    return g.getFontMetrics(font);
  }

  public java.awt.Rectangle getClipBounds() {
      return g.getClipBounds();
  }

  public void clipRect(int x, int y, int w, int h) {
      g.clipRect(x,y,w,h);
  }

  public Shape getClip() {
      return g.getClip();
  }

  public void copyArea(int x, int y, int width, int height, int dx, int dy) {
      g.copyArea(x,y,width,height,dx,dy);
  }

  public boolean drawImage(Image img,
                           int x,
                           int y,
                           ImageObserver observer) {
    return g.drawImage(img,x,y,observer);
  }

  public boolean drawImage(Image img,
                                   int x,
                                   int y,
                                   int w,
                                   int h,
                                   ImageObserver observer) {
    return g.drawImage(img,x,y,w,h,observer);
  }

  public boolean drawImage(Image img,
                                   int x,
                                   int y,
                                   Color bgcolor,
                                   ImageObserver observer) {
    return g.drawImage(img,x,y,bgcolor,observer);
  }

  public boolean drawImage(Image img,
                                   int x,
                                   int y,
                                   int width,
                                   int height,
                                   Color bgcolor,
                                   ImageObserver observer) {
    return g.drawImage(img,x,y,width,height,bgcolor,observer);
  }

  public boolean drawImage(Image img,
                                   int dx1,
                                   int dy1,
                                   int dx2,
                                   int dy2,
                                   int sx1,
                                   int sy1,
                                   int sx2,
                                   int sy2,
                                   ImageObserver observer) {
    return g.drawImage(img,dx1,dy1,dx2,dy2,sx1,sy1,sx2,sy2,observer);
  }

  public boolean drawImage(Image img,
                                   int dx1,
                                   int dy1,
                                   int dx2,
                                   int dy2,
                                   int sx1,
                                   int sy1,
                                   int sx2,
                                   int sy2,
                                   Color bgcolor,
                                   ImageObserver observer) {
    return g.drawImage(img,dx1,dy1,dx2,dy2,sx1,sy1,sx2,sy2,bgcolor,observer);
  }

  public void drawRect(int x, int y, int w, int h) {
      g.drawRect(x,y,w,h);
  }

  public void fillRect(int x, int y, int w, int h) {
      g.fillRect(x,y,w,h);
  }

  public void clearRect(int x, int y, int w, int h) {
      g.clearRect(x,y,w,h);
  }

  public void drawOval(int x, int y, int w, int h) {
      g.drawOval(x,y,w,h);
  }

  public void fillOval(int x, int y, int w, int h) {
      g.fillOval(x,y,w,h);
  }

  public void drawArc(int x, int y, int w, int h, int startAngle, int arcAngle) {
      g.drawArc(x,y,w,h,startAngle,arcAngle);
  }

  public void fillArc(int x, int y, int w, int h, int startAngle, int arcAngle) {
      g.fillArc(x,y,w,h,startAngle,arcAngle);
  }

  public void drawRoundRect(int x, int y, int w, int h, int arcw, int arch) {
      g.drawRoundRect(x,y,w,h,arcw,arch);
  }

  public void fillRoundRect(int x, int y, int w, int h, int arcw, int arch) {
      g.fillRoundRect(x,y,w,h,arcw,arch);
  }

  public void drawPolygon(int xPoints[], int yPoints[], int nPoints) {
      g.drawPolygon(xPoints,yPoints,nPoints);
  }

  public void drawPolygon(Polygon poly) {
      g.drawPolygon(poly);
  }

  public void fillPolygon(int xPoints[], int yPoints[], int nPoints) {
      g.fillPolygon(xPoints,yPoints,nPoints);
  }

  public void fillPolygon(Polygon poly) {
      g.fillPolygon(poly);
  }

  public void drawPolyline(int xPoints[], int yPoints[], int nPoints) {
      g.drawPolyline(xPoints,yPoints,nPoints);
  }

  public void drawLine(int x1, int y1, int x2, int y2) {
      g.drawLine(x1,y1,x2,y2);
  }

  public void setClip(int x, int y, int w, int h) {
      g.setClip(x,y,w,h);
  }

  public void setClip(Shape clip) {
      g.setClip(clip);
  }

  public void translate(int x, int y) {
      g.translate(x,y);
  }

  public void drawString(String text, int x, int y) {
      g.drawString(text,x,y);
  }

  public void drawBytes(byte[] data, int start, int size,
			int x, int y) {
      g.drawBytes(data,start,size,x,y);
  }

  // For Java 1.2 uncomment the following lines:
  //
  // public void drawString(java.text.AttributedCharacterIterator iterator,
  //   int i, int j)
  // {
  //   g.drawString(iterator,i,j);
  // }

}
