package CH.ifa.draw.util;

import java.awt.*;
import java.util.*;

public class ExtendedGraphicsImpl extends ContextGraphics
				  implements ExtendedGraphics {

    /** Extends ContextGraphics by the ability to draw dashed lines.
     */
  public ExtendedGraphicsImpl(Graphics g, Color background) {
      super(g,background);
  }

  /** Draws a dashed line starting at (x1,y1) and ending at (x2,y2)
   *  with the interval size strokeSize. That means a stroke of length
   *  strokeSize is followed by a gap of length strokeSize and so on.
   */
  public void drawDashedLine(int x1, int y1, int x2, int y2, int strokeSize) {
      drawDashedLine(x1,y1,x2,y2,strokeSize,5);
  }


 /* Draws a dashed Polyline (see drawDashedLine() and drawPolyline()).
  */
  public void drawDashedPolyline(int xPoints[], int yPoints[], int nPoints,
				 int strokeSize) {
      if (nPoints>=2) {
	  int offset=0;
	  int x=xPoints[0],y=yPoints[0];
	  for (int i=1; i<nPoints; ++i) {
	    int x1=x, y1=y;
	    x=xPoints[i]; y=yPoints[i];
	    offset=drawDashedLine(x1,y1,x,y,strokeSize,offset);
	  }
      }
  }

 /* Draws a dashed Polyline (see drawDashedLine() and drawPolyline()).
  */
  public void drawDashedPolyline(Vector points, int strokeSize) {
      int index=points.size();
      if (index>=2) {
	  int offset=0;
	  Point old=(Point)points.elementAt(--index);
	  while (index>0) {
	    Point next=(Point)points.elementAt(--index);
	    offset=drawDashedLine(old.x,old.y,next.x,next.y,strokeSize,offset);
	    old=next;
	  } 
      }
  }

  private int drawDashedLine(int x1, int y1, int x2, int y2, int strokeSize,
			     int offset) {
    double alpha=Math.atan2(x2-x1,y2-y1);
    double dx=Math.sin(alpha),
	   dy=Math.cos(alpha);
    double length=Geom.length(x1,y1, x2,y2);
    int s=offset;
    while (s<length) {
	int xa,ya,xb,yb;
	if (s<=0) {
	    xa=x1;
	    ya=y1;
	} else {
	    xa=x1+(int)Math.round(s*dx);
	    ya=y1+(int)Math.round(s*dy);
	}
	s+=strokeSize;
	if (s>=length) {
	    xb=x2;
	    yb=y2;
	} else {
	    xb=x1+(int)Math.round(s*dx);
	    yb=y1+(int)Math.round(s*dy);
	}
	s+=strokeSize;
	drawLine(xa,ya,xb,yb);
    }
    s-=length;
    if (s>=strokeSize)
	s-=2*strokeSize;
    return s;
  }

  /** Additionally supports underlined text. */
  public void drawString(String text, int x, int y) {
    if (text.length()>0) {
      g.drawString(text,x,y);
      // for special fonts like ZapfDingbats, better use:
      //g.drawBytes(text.getBytes(),0,line.length(),x,y);
      Font font=g.getFont();
      if (font instanceof ExtendedFont &&
	((ExtendedFont)font).isUnderlined()) {
	int pixel=g.getFont().getSize()/12;
	if (pixel==0) pixel=1;
	y+=pixel;
	g.fillRect(x, y, g.getFontMetrics().stringWidth(text), pixel);
      }
    }
  }

}
