package CH.ifa.draw.util;

import java.awt.*;
import java.awt.event.*;
import CH.ifa.draw.util.Fontkit;

/**
 * MessageDialog: A utility class for building simple modal dialog windows
 * including a multi-line message text and a set of user buttons.
 *
 */

public class MessageDialog implements ActionListener {
  public final static int LEFT = 0;
  public final static int CENTER = 1;
  public final static String SPACE = "  ";
  public final static String DEFAULT_BUTTON_LABEL = " OK ";
  private Button[] button;
  private Panel textPanel, buttonPanel;
  private Dialog alert;
  private int pressedButton;
  
  // Die Zeilen der MessageDialog werden als Felder eines String-Arrays ?bergeben.
  // z.B. MessageDialog a = new MessageDialog(this, "Titel", new String[] {"Zeile 1","Zweite Zeile"});
  
  // Aufruf mit Titel und Array
  MessageDialog(Frame f, String title, String[] message) {
    this(f, title, message, DEFAULT_BUTTON_LABEL);
  }
  
  // Aufruf mit Titel, Array und einem Buttonlabel
  MessageDialog(Frame f, String title, String[] message, String label) {
    this(f, title, message, CENTER, new String[]{label});
  }

  // Aufruf mit Titel, Array und Buttonlabels
  MessageDialog(Frame f, String title, String[] message, String[] label) {
    this(f, title, message, CENTER, label);
  }

  // Aufruf mit allen Parametern
  MessageDialog(Frame f, String title, String[] message, int alignment, String[] label) {
    alert = new Dialog(f, title, true);
    alert.setLayout(new BorderLayout());
    alert.setBackground(Color.lightGray);
    
    // Einen Panel f?r den Text
    textPanel = new Panel();
    textPanel.setLayout(new GridLayout(0, 1));
    int alignmentValue = alignment == LEFT ? Label.LEFT : Label.CENTER;
    for (int i=0; i<message.length; i++)
      {
        // Die Textzeilen als "Label"s erzeugen. Font setzen.
        // System.out.println(message[i]);  // Debugging
        Label l = new Label(SPACE+message[i]+SPACE, alignmentValue);
        l.setFont(Fontkit.getFont("Serif", Font.BOLD, 13));
        textPanel.add(l);
      }
    
    // Ein Panel f?r die Buttons, da sie zentriert werden sollen.
    buttonPanel = new Panel();
    buttonPanel.setLayout(new FlowLayout());
    // Die Buttons
    button = new Button[label.length];
    for (int i=0; i<label.length; ++i) {
      button[i] = new Button(label[i]);    
      button[i].setActionCommand(String.valueOf(i));
      button[i].addActionListener(this);
      buttonPanel.add(button[i]);
    } /* endfor */

    // Die Panels hinzuf?gen
    alert.add("Center", textPanel);
    alert.add("South", buttonPanel);
    alert.pack();
    alert.setLocation(330,230);
  }

  public static int invokeDialog(Frame f, String title, 
          String[] message, int alignment, String[] label) {
      MessageDialog md = new MessageDialog(f,title,message,alignment,label);
      md.invoke();
      md.alert.dispose();
      return md.pressedButton;
  }


  public int invoke() {  // spter: Parameter int defaultButton
        pressedButton = 2;
        alert.setVisible(true);
        // ...is automatically suspended until alert closed!
        return pressedButton;
  }

  // Wenn der OK-Button gedr?ckt wird, dann soll die Alert-Box geschlossen werden.
  public void actionPerformed(ActionEvent e) 
    {
      //System.out.println("actionPerformed: "+e.getActionCommand());
      pressedButton = Integer.valueOf(e.getActionCommand()).intValue();
      alert.setVisible(false);              // Dialog entfernen
    }


}


