package de.renew.application;

import de.renew.simulator.*;
import de.renew.shadow.*;
import java.io.*;

public class ShadowSimulator {
  public static void main(String[] args) {
    if (args.length!=3) {
      System.out.println("Usage: java de.renew.application.ShadowSimulator "+
	"SIMCLASS FILE NETNAME");
      System.exit(0);
    }
    
    Simulator simulator=null;
    try {
      Class clazz=Class.forName(args[0]); // No reload required.
      simulator=(Simulator)clazz.newInstance();
    } catch (Exception e) {
      System.out.println("Could not load simulator class "+args[0]+".");
      System.exit(0);
    }

    ShadowNetSystem netSystem=null;
    try {
      FileInputStream stream=new FileInputStream(args[1]);
      ObjectInput input=new ObjectInputStream(stream);
      netSystem=(ShadowNetSystem)input.readObject();
    } catch (Exception e) {
      e.printStackTrace();
      System.out.println("Could not load net system from file "+args[1]+".");
      System.exit(0);
    }

    ShadowLookup lookup=null;
    try {
      lookup=netSystem.compile();
    } catch (Exception e) {
      e.printStackTrace();
    }
    if (lookup==null) {
      System.out.println("Could not compile net system.");
      System.exit(0);
    }

    Net net=lookup.getNet(args[2]);
    if (net==null) {
      System.out.println("Net system does not contain the net "+args[2]+".");
      System.exit(0);
    }

    NetInstance primaryInstance=SimulatorHelper.fireInitially(net,
      new ConsoleTracer("Initially","(0) "),true);

    if (primaryInstance!=null) {
      // The simulator can now get enabled transitions from the search queue.
      simulator.startRun();
    } else {
      System.out.println("The net did not accept the init request.");
    }
   
  }
}
