package de.renew.formalism.fsnet;

// We need an explicit override for the automatically generated
// classes, otherwise we get serious type errors.
import de.renew.formalism.java.ParseException;
import de.renew.formalism.java.Token;
import de.renew.shadow.SyntaxException;
import de.renew.shadow.*;
import collections.*;
import java.util.*;
import java.util.Enumeration;
import de.uni_hamburg.fs.*;
import de.renew.formalism.java.*;
import de.renew.simulator.*;
import de.renew.expression.*;
import de.renew.formalism.fs.*;
import de.renew.util.StringUtil;

public class XFSNetCompiler extends FSNetCompiler {
    //static final long serialVersionUID = 262012294953452116L;

  public static final String RESULT="RESULT";
  public static final String VAR_PREFIX="v";

  protected InscriptionParser makeParser(String inscr) {
    return new XFSNetParser(new java.io.StringReader(inscr));
  }

  protected CollectionEnumeration parseArcInscription(String inscr,
    ParsedDeclarationNode declaration) throws SyntaxException
  {
    if (inscr==null || inscr.equals("")) {
      inscr="#9999";
    }
    return super.parseArcInscription(inscr,declaration);
  }

  protected void compileTransitionInscriptions(
       ShadowTransition shadowTransition,
       ParsedDeclarationNode declaration,
       Vector parsedInscriptions, Vector errorShadows)
      throws SyntaxException {
System.out.println("\nCompiling Transition "+shadowTransition.getName());
    FSShadowLookup lookup=(FSShadowLookup)getLookup();
    // Transition already exists.
    // Only insert the inscriptions.
    varcnt=0; // reset local variable counter
    TransitionRule transitionRule=new TransitionRule();
    transitionRule.append("[Any\n");
    CollectionEnumeration inscriptions=shadowTransition.elements();
    Vector inputArcVars=new Vector(), // of Number-Strings ("0","1",..)
           transVars=new Vector(); // of Var-Names (Name("v0"),Name("v1")...
    while (inscriptions.hasMoreElements()) {
      Object elem=inscriptions.nextElement();
      if (elem instanceof ShadowInscription) {
	ShadowInscription inscription=(ShadowInscription)elem;
	if (!inscription.inscr.startsWith(" ")) {
	    String transVar=newVariable();
	    transVars.addElement(transVar);
	    transitionRule.append(transVar,inscription.inscr,inscription);
	} else {
	    // JavaNet inscription:
	System.out.println(inscription.inscr.substring(1)+";");
	    CollectionEnumeration subinscriptions=null;
	    try {
	      subinscriptions=makeInscriptions(inscription.inscr.substring(1),
					       lookup,true,declaration);
	    } catch (SyntaxException e) {
	      throw e.addObject(inscription);
	    }
	    
	    while (subinscriptions.hasMoreElements()) {
	      parsedInscriptions.addElement(subinscriptions.nextElement());
	      errorShadows.addElement(inscription);
	    }
	}
      } else if (elem instanceof ShadowArc) {
	  compileSingleArc((ShadowArc)elem,declaration,transitionRule,
			   inputArcVars,parsedInscriptions);
      } else {
	  super.compileTransitionInscriptions(shadowTransition,
					      declaration,
					      parsedInscriptions,
					      errorShadows);
      }
    }
    transitionRule.append("]");
    XFSNetParser parser=(XFSNetParser)makeParser(transitionRule.toString());
    parser.setLookup(lookup);
    parser.setDeclarationNode(declaration);

    try {
      FSUnifyExpression fsExpr=parser.TransitionRule();
      //System.out.println("Parsed transition rule TMP0:\n"+fsExpr.getTemplate());
      // Look for Up- and Downlinks:
      Node fs=fsExpr.getTemplate().getRoot();
      String uplinkVar=null;
      int dlVarCnt=0;
      Enumeration tVarEnum=transVars.elements();
      while (tVarEnum.hasMoreElements()) {
	  String transVar=(String)tVarEnum.nextElement();
	  Name feature=new Name(VAR_PREFIX+transVar);
	  // for each Transition inscription feature:
	  Node subfs=fs.delta(feature);
	  Type subfsType=subfs.getType();
	  if (subfsType.getName().equals(LINK)) {
	      if (!subfs.hasFeature(RCV)) {
		  // is Uplink:
		  //System.out.println("Uplink found at "+feature+": "+new FeatureStructure(subfs));
		  if (uplinkVar!=null)
		      throw new SyntaxException(
			     "Transition has more than one uplink.",null);
		  uplinkVar=transVar;
		  inputArcVars.addElement(transVar);
	      } else {
		  // is Downlink:
		  addDownlink(parsedInscriptions,fsExpr,transVar,
			      dlVarCnt++,new Path(feature));
	      }
	  } else if (subfsType instanceof ListType) {
	      ListType subListType=(ListType)subfsType;
	      if (subListType.getBaseType().getName().equals(LINK)) {
		  // is Downlink-List:
		  Path listPath=new Path(feature);
		  while (((ListType)fs.delta(listPath).getType())
			  .getSubtype()==ListType.NELIST) {
		      Path dlPath=listPath.append(ListType.HEAD);
		      addDownlink(parsedInscriptions,fsExpr,transVar,
				  dlVarCnt++,dlPath);
		      listPath=listPath.append(ListType.TAIL);
		  }
	      }
	  }
      }
      if (uplinkVar!=null) {
	  Path uplinkPath=new Path(VAR_PREFIX+uplinkVar+":"+PARAM);
	  System.out.println(":s(v"+uplinkVar+",TMP"+dlVarCnt+atExpr(uplinkPath)+",UL);");
	  parsedInscriptions.addElement(
	     new UplinkInscription("s",getTriple(
	       getVariableExpression(uplinkVar),
	       getFSAtExpression(getVariableExpression("TMP"+dlVarCnt),
				 uplinkPath),
	       getVariableExpression("UL")
	       ))
	     );
	  System.out.println("TMP"+(dlVarCnt+1)+" = TMP"+dlVarCnt
			     +".unify(UL,\""+uplinkPath+"\");");
	  parsedInscriptions.addElement(
	    new ExpressionInscription(
	      new EqualsExpression(
	        FeatureStructure.class,
		getVariableExpression("TMP"+(dlVarCnt+1)),
		new CallExpression(
		  FeatureStructure.class,
		  new TupleExpression(new Expression[]{
		    getVariableExpression("TMP"+dlVarCnt),
		    getVariableExpression("UL")}),
		  new FSUnifyAtFunction(uplinkPath)))));
	  dlVarCnt++;
      }
      System.out.println("action RESULT = TMP"+dlVarCnt+".instantiate();");
      parsedInscriptions.addElement(
	new ActionInscription(
	  new EqualsExpression(
	    FeatureStructure.class,
	    getVariableExpression(RESULT),
	    new CallExpression(FeatureStructure.class,
			       getVariableExpression("TMP"+dlVarCnt),
			       InstantiateFunction.INSTANCE))));

      System.out.println("guard TMP"+dlVarCnt+".canInstantiate();");
      parsedInscriptions.addElement(
	new GuardInscription(
	  new CallExpression(Boolean.TYPE,
			     getVariableExpression("TMP"+dlVarCnt),
			     CanInstantiateFunction.INSTANCE)));


System.out.println("TMP0="+transitionRule+"\n+[Any");
      Enumeration varenum=inputArcVars.elements();
      while (varenum.hasMoreElements()) {
	  String var=(String)varenum.nextElement();
	  String varpath=VAR_PREFIX+var;
	  if (var.equals(uplinkVar))
	     varpath+=":"+PARAM;
System.out.println(" "+varpath+":(v"+var+")");
	  fsExpr.getPaths().addElement(new Path(varpath));
	  fsExpr.getExprs().addElement(new VariableExpression(FeatureStructure.class,
							new LocalVariable(var)));
      }
System.out.println("];");
      // Build TMP0=<transitionRule>:
      parsedInscriptions.addElement
	  (new ExpressionInscription
	   (new EqualsExpression(FeatureStructure.class,
				 getVariableExpression("TMP0"),
				 fsExpr)));
    } catch (de.renew.formalism.java.ParseException e) {
      throw makeSyntaxException(e,shadowTransition,transitionRule.inscrAreas);
    }

  }

  private void addDownlink(Vector parsedInscriptions,
			   FSUnifyExpression fsExpr,
			   String transVar, int dlVarCnt,
			   Path dlpath) {
      //System.out.println("Downlink found at "+dlpath); //+": "+fsExpr.template.at(dlpath));
      String prevTmpVar="TMP"+String.valueOf(dlVarCnt);
      String paramVar="DL"+String.valueOf(++dlVarCnt);
      Path rcvpath=dlpath.append(RCV);
      Path parampath=dlpath.append(PARAM);
      // Build Downlink:
      System.out.println(prevTmpVar+atExpr(rcvpath)
			 +": s("+prevTmpVar+atExpr(parampath)+","
			 +paramVar+",RESULT"+atExpr(parampath)+");");
      parsedInscriptions.addElement(new DownlinkInscription(
        "s",
	getTriple(getFSAtExpression(getVariableExpression(prevTmpVar),
				    parampath),
		  getVariableExpression(paramVar),
		  getFSAtExpression(getVariableExpression(RESULT),parampath)),
	getFSAtExpression(getVariableExpression(prevTmpVar),rcvpath,true),
	false));

      String tmpVar="TMP"+dlVarCnt;
      // Build EqualsExpresion:
      System.out.println(tmpVar+"="+prevTmpVar
			 +".unify("+paramVar+",\""+parampath+"\");");
      parsedInscriptions.addElement(
	new ExpressionInscription(
	  new EqualsExpression(FeatureStructure.class,
			       getVariableExpression(tmpVar),
			       new CallExpression(
				 FeatureStructure.class,
				 new TupleExpression(new Expression[]{
				   getVariableExpression(prevTmpVar),
				   getVariableExpression(paramVar)}),
				 new FSUnifyAtFunction(parampath)))));
  }

  private static String atExpr(Path path) {
      return "@"+path.toString().replace(':','@');
  }

  private static int varcnt=0;

  private static String newVariable() {
      return String.valueOf(varcnt++);
  }


  private VariableExpression getVariableExpression(String varName) {
      return new VariableExpression(FeatureStructure.class,
				    new LocalVariable(varName));
  }

  private CallExpression getFSAtExpression(Expression expr, Path path) {
     return getFSAtExpression(expr,path,false);
  }

  private CallExpression getFSAtExpression(Expression expr, Path path,
					   boolean unpack) {
      return new CallExpression(FeatureStructure.class,
				expr, new FSAtFunction(path,unpack));
  }

  private TupleExpression getTriple(Expression c1, Expression c2,
				    Expression c3) {
      return new TupleExpression(new Expression[]{c1,c2,c3});
  }

  private void compileSingleArc(ShadowArc shadowArc,
				ParsedDeclarationNode declaration,
				TransitionRule transitionRule,
				Vector vars,
				Vector parsedInscriptions)
                 throws SyntaxException {
    Enumeration arcinscrs=shadowArc.elements();
    if (arcinscrs.hasMoreElements()) {
System.out.println("Type:"+shadowArc.place.getName());
      do {
	ShadowInscription inscr=(ShadowInscription)arcinscrs.nextElement();
	compileArcInscription(inscr, inscr.inscr, shadowArc, declaration,
			      transitionRule, vars, parsedInscriptions);
      } while (arcinscrs.hasMoreElements());
    } else {
      compileArcInscription(null, "#9999", shadowArc, declaration,
			    transitionRule, vars, parsedInscriptions);
    }
  }

  private void compileArcInscription(ShadowInscription si,
				     String inscr,
				     ShadowArc shadowArc,
				     ParsedDeclarationNode declaration,
				     TransitionRule transitionRule,
				     Vector vars,
				     Vector parsedInscriptions)
                 throws SyntaxException {
    FSShadowLookup lookup=(FSShadowLookup)getLookup();
    //System.out.println("Found arc from "+lookup.get(shadowArc.place).getName());
    String var=newVariable();
    //System.out.println("  Arcinscription "+inscr+" mapped to "+var);
    transitionRule.append(var,inscr,si);
    if (shadowArc.shadowArcType==ShadowArc.inhibitor
	|| (shadowArc.shadowArcType==ShadowArc.ordinary
	    && !shadowArc.placeToTransition)) {
      String vvar=VAR_PREFIX+var;
System.out.println("action "+vvar+" = RESULT@"+vvar+";");
      parsedInscriptions.addElement(
	new ActionInscription(
	  new EqualsExpression(
	    FeatureStructure.class,
	    getVariableExpression(var),
	    getFSAtExpression(getVariableExpression(RESULT),
			      new Path(vvar),true))));
    } else {
	vars.addElement(var);
    }
    getArcFactory(shadowArc)
	.compileArc(lookup.get(shadowArc.place),
		    lookup.get(shadowArc.transition),
		    shadowArc.getTrace(),
		    FeatureStructure.class,
		    new TimedExpression
		    (new TypedExpression(FeatureStructure.class,
					 getVariableExpression(var)),null));
  }

  public static SyntaxException makeSyntaxException(ParseException pe,
						    ShadowTransition trans,
						    Vector sias) {
    ShadowInscriptionArea sia=null;
    if (pe.currentToken!=null) {
	Token posToken=pe.currentToken;
	if (posToken.next!=null)
	    posToken=posToken.next;
	int line=posToken.beginLine;
	//System.out.println("Looking for line "+line);
	Enumeration siaelems=sias.elements();
	while (siaelems.hasMoreElements()) {
	    sia=(ShadowInscriptionArea)siaelems.nextElement();
	    //System.out.println("Found range "+sia.startLine+"-"+sia.lastLine);
	    if (sia.matches(line))
		break;
	    sia=null;
	}
	if (sia!=null) {
	    posToken.beginLine-=sia.startLine-1;
        }
    }
    SyntaxException se=makeSyntaxException(pe);
    if (sia!=null) {
	se.addObject(sia.inscription);
    } else {
	se.addObject(trans);
    }
    return se;
  }

}

class ShadowInscriptionArea {
   int startLine, lastLine;
   ShadowInscription inscription;

   ShadowInscriptionArea(int startLine, int lastLine,
                         ShadowInscription inscription) {
       this.startLine=startLine;
       this.inscription=inscription;
       this.lastLine=lastLine;
   }

    boolean matches(int line) {
	return startLine<=line && line <=lastLine;
    }

}

class TransitionRule {
  StringBuffer fs=new StringBuffer();
  Vector inscrAreas=new Vector();
  int line=1;

  void append(String partFS) {
    fs.append(partFS);
    line+=StringUtil.countLines(partFS)-1;
  }

  void append(String var, String inscr, ShadowInscription si) {
    fs.append(XFSNetCompiler.VAR_PREFIX).append(var).append(":\n");
    int startLine=++line;
    append(inscr+"\n");
    if (si!=null) {
      ShadowInscriptionArea sia=new ShadowInscriptionArea(startLine,line-1,si);
      inscrAreas.addElement(sia);
    }
  }

  public String toString() {
    return fs.toString();
  }

}
