package de.renew.formalism.java;

import de.renew.simulator.*;
import de.renew.expression.Expression;
import de.renew.shadow.*;
import de.renew.function.*;
import de.renew.util.Types;
import de.renew.unify.List;
import java.util.Vector;
import java.util.Enumeration;

public class FlexibleOutArcFactory implements ArcFactory {
  public static FlexibleOutArcFactory INSTANCE=new FlexibleOutArcFactory ();

  private FlexibleOutArcFactory() {}

  public void emptyArcCheck() throws SyntaxException {
    throw new SyntaxException("Flexible arcs must be inscribed.",null);
  }

  public boolean allowsTime() {
    return true;
  }

  public void compileArc(Place place, Transition transition, boolean trace,
    Class placeType, TimedExpression timedExpr) throws SyntaxException
  {
    TypedExpression typedExpr=timedExpr.getExpression();

    Function conversionFunction=null;

    Class exprType=typedExpr.getType();
    if (exprType!=Types.UNTYPED) {
      if (exprType==null) {
	throw new SyntaxException("Null not allowed for flexible arcs.",null);
      } else if (List.class.equals(exprType) ||
		 Enumeration.class.equals(exprType) ||
		 Vector.class.equals(exprType))
      {
	// Check for untyped output place.
	if (placeType!=Types.UNTYPED) {
	  throw new SyntaxException(
            "For non-array inscriptions the place must be untyped.",null);
	}
      } else if (!exprType.isArray()) {
	throw new SyntaxException(
          "Incorrect type for flexible arc inscription.",null);
      }
    }

    if (placeType==Types.UNTYPED) {
      // Nothing to do. We have no indication that the
      // elements transported by the flexible arc must be converted
      // to a different type. No type checking is required.
    } else if (!typedExpr.isTyped()) {
      throw new SyntaxException(
	"Output arc expression for typed place must be typed.",null);
    } else {
      // We already know that the expression is of array type.
      Class elementType=exprType.getComponentType();

      if (!Types.allowsLosslessWidening(elementType,placeType)) {
	throw new SyntaxException("Cannot losslessly convert "+
	  JavaHelper.makeTypeErrorString(elementType)+
	  " to "+JavaHelper.makeTypeErrorString(placeType)+".",
          null);
      }

      if (placeType.isPrimitive()) {
	conversionFunction=new CastFunction(placeType);
      }
    }

    FlexibleArc arc=new FlexibleArc(place,FlexibleArc.out,
      typedExpr.getExpression(),conversionFunction,null);
    arc.setTrace(trace);
    transition.add(arc);
  }
}
