package de.renew.function;

import java.lang.reflect.*;
import de.renew.unify.*;
import de.renew.util.Value;

import de.renew.util.ReflectionSerializer;
import java.io.IOException;


public class StaticFieldFunction implements NoArgFunction {

  /** 
   * This field is not really transient, but as the reflection
   * classes are not serializable, we have to store it by
   * ourselves.
   **/
  transient private Field field;

  public StaticFieldFunction(Field field) {
    this.field=field;
  }

  public Object function() throws Impossible {
    try {
      return Value.possiblyWrap(field.get(null),
	field.getType().isPrimitive());
    }
    catch (Exception e) {
      throw new Impossible();
    }
  }

  /**
   * Serialization method, behaves like default writeObject
   * method. Stores the not-really-transient field field.
   **/
  private void writeObject(java.io.ObjectOutputStream out)
      throws IOException {
      out.defaultWriteObject();
      ReflectionSerializer.writeField(out, field);
  }

  /**
   * Deserialization method, behaves like default readObject
   * method. Restores the not-really-transient field field.
   **/
  private void readObject(java.io.ObjectInputStream in)
      throws IOException, ClassNotFoundException {
      in.defaultReadObject();
      field = ReflectionSerializer.readField(in);
  }

}
