package de.renew.gui;

import de.renew.simulator.TransitionInstance;
import de.renew.simulator.InscriptionOccurrence;
import de.renew.simulator.Searcher;
import de.renew.expression.VariableMapper;
import de.renew.unify.Copier;
import de.renew.unify.Impossible;

import collections.CollectionEnumeration;
import de.renew.util.SingletonEnumeration;
import de.renew.util.EmptyEnumeration;

/**
 * Occurrence of a breakpoint inscription.
 *
 * BreakpointOccurrence.java
 * Created: Wed May 31  2000
 * @author Michael Duvigneau
 *
 * @see BreakpointInscription
 **/
class BreakpointOccurrence  implements InscriptionOccurrence  {
  
  private GlobalTransitionBreakpoint breakpoint;

  /**
   * The transition instance this inscription occurs at.
   * The breakpoint has to know at which transition instance
   * it occurrs to be able to inform the user.
   **/
  private TransitionInstance instance;

  BreakpointOccurrence(GlobalTransitionBreakpoint breakpoint,
		       TransitionInstance instance) {
    this.breakpoint = breakpoint;
    this.instance = instance;
  }

  /**
   * @return    empty enumeration
   *            (breakpoint holds in all bindings)
   * @exception de.renew.unify.Impossible
   *            never?
   */
  public CollectionEnumeration makeBinders(Searcher searcher) throws Impossible {
    return new EmptyEnumeration();
  }

  /**
   * @return one executable of this breakpoint
   * @see BreakpointExecutable
   **/
  public CollectionEnumeration makeExecutables(Copier copier,
					       VariableMapper mapper) {
    return new SingletonEnumeration(new BreakpointExecutable(breakpoint, instance));
  }
  
}
