/*
 * @(#)CPNApplication.java 5.1
 *
 */

package de.renew.gui;

import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import java.awt.*;

public  class CPNDrawingView extends StandardDrawingView {
  private Object repaintLock=new Object();
  private transient Rectangle damage = null;
  private transient Rectangle committedDamage = null;
  private transient boolean repaintActive=false;

  public CPNDrawingView(DrawingEditor editor, int width, int height) {
    super(editor,width,height);
    setBackground(Color.white);
  }

  public void drawingInvalidated(DrawingChangeEvent e) {
    synchronized(repaintLock) {
      Rectangle r = e.getInvalidatedRectangle();
      if (r!=null) {
	if (damage == null)
	  damage = new Rectangle(r);
	else
	  damage.add(r);
      }
    }
  }

  public void repairDamage() {
    synchronized(repaintLock) {
      // Take note of additional request.
      if (committedDamage == null)
	committedDamage = damage;
      else if (damage != null) {
	committedDamage.add(damage);
      }
      damage=null;

      if (!repaintActive) {
	if (committedDamage!=null) {
	  repaintActive=true;
	  repaint(committedDamage.x,committedDamage.y,
	    committedDamage.width,committedDamage.height);
	  committedDamage=null;
	}
      }
    }
  }

  public void paint(Graphics g) {
    // We are currently updating the view, so further
    // updates should wait a while.
    synchronized(repaintLock) {
      repaintActive=true;
    }

    super.paint(g);

    synchronized(repaintLock) {
      // Any comitted damage?
      if (committedDamage!=null) {
	// Yes, handle the requested repaint in a separate
	// request. We cannot do the repaint directly, because
	// our graphics context might have an inappropriate
	// clip area.
	repaint(committedDamage.x,committedDamage.y,
	  committedDamage.width,committedDamage.height);
	committedDamage=null;
      } else {
	// No, new damage must start a repaint explicitly.
	repaintActive=false;
      }
    }
  }

}
