/*
 * @(#)CircleDecoration.java 5.1
 *
 */

package de.renew.gui;

import java.io.*;
import java.awt.*;

import CH.ifa.draw.framework.*;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.util.*;

/**
 * A circle line decoration.
 * @see CH.ifa.draw.figures.PolyLineFigure
 */
public  class CircleDecoration implements LineDecoration {

    private double fRadius;
    private boolean fFilled;

    /*
     * Serialization support.
     */
    private static final long serialVersionUID = -3459870923049638L;
    private int circleDecorationSerializedDataVersion = 1;

    public CircleDecoration() {
      fRadius = 4;
      fFilled = true;
    }

   /**
    * Draws the circle in the direction specified by the given two
    * points.
    */
    public void draw(Graphics g, int x1, int y1, int x2, int y2,
      Color fillColor, Color lineColor)
    {
      double dx=x2-x1;
      double dy=y2-y1;
      double rr=dx*dx+dy*dy;
      if (rr!=0) {
	// distinct points
	double r=Math.sqrt(rr);
	x1+=(int)(dx*fRadius/r);
	y1+=(int)(dy*fRadius/r);
      }

      // if filled, fill with lineColor.
      if (fFilled) {
	fillColor=lineColor;
      }
      
      if (!ColorMap.isTransparent(fillColor)) {
	g.setColor(fillColor);
	g.fillOval((int)(x1-fRadius),(int)(y1-fRadius),
	  (int)(2*fRadius),(int)(2*fRadius));
      }
      
      // if not filled, outline with lineColor.
      if (!fFilled && !ColorMap.isTransparent(lineColor)) {
	g.setColor(lineColor);
	g.drawOval((int)(x1-fRadius),(int)(y1-fRadius),
	  (int)(2*fRadius),(int)(2*fRadius));
      } 
    }

    /**
     * Stores the arrow tip to a StorableOutput.
     */
    public void write(StorableOutput dw) {
    }

    /**
     * Reads the arrow tip from a StorableInput.
     */
    public void read(StorableInput dr) throws IOException {
    }

}
