package de.renew.gui;

import de.renew.shadow.*;
import de.renew.util.SelectiveClassLoader;
import de.renew.util.ClassSource;

public class ClassSourceResetCompiler implements ShadowCompiler {
  private ShadowCompiler compiler;
  public ClassSourceResetCompiler(ShadowCompiler compiler) {
    this.compiler=compiler;
  }

  protected static void setupClassSource() {
    SelectiveClassLoader classLoader=new SelectiveClassLoader();
    classLoader.setSelectors(new String[]{
      "de.renew.util.ReloadableDeserializerImpl",
      "-java",
      "-collections.",
      "-CH.ifa.draw.",
      "-de.renew.",
      "-de.uni_hamburg.fs."
    });
    de.renew.util.ClassSource.setClassLoader(classLoader);
  }
  
  public ShadowLookup compile(ShadowNetSystem netSystem)
    throws SyntaxException
  {
    setupClassSource();
    return compiler.compile(netSystem);
  }

  public String[] getLintNames() {
    return compiler.getLintNames();
  }

  public void lint(int lintNr, ShadowNetSystem netSystem,
    ShadowNet selectedNet, ShadowNet lastSelectedNet)
    throws SyntaxException
  {
    setupClassSource();
    compiler.lint(lintNr,netSystem,selectedNet,lastSelectedNet);
  }

  public String checkDeclarationNode(String inscr, boolean special)
    throws SyntaxException
  {
    setupClassSource();
    return compiler.checkDeclarationNode(inscr,special);
  }

  public String checkArcInscription(String inscr, boolean special,
				    ShadowNet shadowNet)
    throws SyntaxException
  {
    setupClassSource();
    return compiler.checkArcInscription(inscr,special,shadowNet);
  }

  public String checkTransitionInscription(String inscr, boolean special,
					   ShadowNet shadowNet)
    throws SyntaxException
  {
    setupClassSource();
    return compiler.checkTransitionInscription(inscr,special,shadowNet);
  }

  public String checkPlaceInscription(String inscr, boolean special,
				      ShadowNet shadowNet)
    throws SyntaxException 
  {
    setupClassSource();
    return compiler.checkPlaceInscription(inscr,special,shadowNet);
  }
}
