package de.renew.gui;

import CH.ifa.draw.framework.*;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.util.Palette;
import de.renew.unify.Aggregate;
import de.renew.formalism.java.*;
import de.renew.shadow.*;
import de.renew.simulator.NetInstance;
import de.renew.util.TextToken;
import java.awt.*;
import java.util.*;

public class JavaMode implements RenewMode {

  CPNSimulation fSimulation=null;

  public ShadowCompiler getCompiler() {
      return new JavaNetCompiler(false,false,false);
  }

  public CPNSimulation getSimulation() {
      if (fSimulation==null)
	  fSimulation=new CPNSimulation(getCompiler(),false);
      return fSimulation;
  }

  public CPNTextFigure newInscriptionFigure(InscribableFigure figure) {
      return new CPNTextFigure();
  }

  public String getDefaultInscription(InscribableFigure figure) {
      if (figure instanceof ArcConnection)
	  return "x";
      if (figure instanceof PlaceFigure)
	  return "[]";
      if (figure instanceof TransitionFigure)
	  return ":s()";
      return "";
  }

  public Figure getTokenFigure(Object token, boolean expanded)
   {
      if (token instanceof Aggregate) {
	  return new AggregateFigure(this,(Aggregate)token,expanded);
      }
      if (token instanceof NetInstance) {
	  NetInstance instance=(NetInstance)token;
	  CPNDrawing drawing=getSimulation().getDrawing(instance.getNet()
							.toString());
	  Figure iconFigure = null;
	  if (drawing != null)
	      iconFigure = drawing.getIconFigure();
	  if (iconFigure!=null) {
	      iconFigure=(Figure)iconFigure.clone();
	      updateID(iconFigure,instance);
	      return iconFigure;
	  }
      }
      if (expanded &&
          !(token instanceof de.uni_hamburg.fs.FeatureStructure ||
	    token instanceof de.renew.simulator.NetInstance ||
	    token instanceof de.renew.util.Value ||
	    token instanceof String))
 	return new de.renew.gui.fs.FSFigure(token);
      else
	return new TextFigure(objectToString(token),true);
   }

   public static String objectToString(Object token) {
      StringBuffer output=new StringBuffer();
      if (token instanceof String) {
	  output.append('"').append((String)token).append('"');
      } else if (token instanceof TextToken) {
	  output.append(((TextToken)token).toTokenText());
      } else {
	  output.append(token);
      }
      return output.toString();
   }

   private void updateID(Figure figure, NetInstance instance) {
       if (figure instanceof TextHolder) {
	   TextHolder textFig=(TextHolder)figure;
	   textFig.setReadOnly(true);
	   textFig.setText(CPNInstanceDrawing.expandMacro(textFig.getText(),
							  instance));
       } else if (figure instanceof CompositeFigure) {
	   // search recursively:
	   FigureEnumeration figenum=figure.figures();
	   while (figenum.hasMoreElements()) {
	       updateID(figenum.nextFigure(),instance);
	   }
       }
   }


  public void createAdditionalTools(CPNApplication editor, Panel toolPanel,
				    Palette palette) {
  }	

  public void createAdditionalMenus(CPNApplication editor, MenuBar menu) {
  }

  public Tool createPlaceTool(DrawingEditor editor) {
    // Delegate back to drawing editor.
    return null;
  }

  public Tool createTransitionTool(DrawingEditor editor) {
    // Delegate back to drawing editor.
    return null;
  }

  public boolean wantClock() {
    return false;
  }

  public boolean wantAux() {
    return false;
  }

  public String getAuxFigureName() {
    return null;
  }

  public String getAuxToolName() {
    return null;
  }

}
