package de.renew.gui;

import CH.ifa.draw.framework.*;
import de.renew.shadow.*;
import de.renew.simulator.Arc;
import java.util.*;
import java.io.*;
import java.awt.*;
import collections.*;

public class ShadowNetSystemRenderer {

  public static ShadowNetSystem readShadowNetSystem(String filename) {
      ShadowNetSystem netSystem=null;
      try {
	  FileInputStream stream=new FileInputStream(filename);
	  ObjectInput input=new ObjectInputStream(stream);
	  netSystem=(ShadowNetSystem)input.readObject();
      } catch (Exception e) {
	  e.printStackTrace();
	  System.out.println("Could not load net system from file "+filename+".");
      }
      return netSystem;
  }

  public static CPNDrawing[] render(String filename) {
      ShadowNetSystem netSystem=readShadowNetSystem(filename);
      CollectionEnumeration netenum=netSystem.elements();
      CPNDrawing[] drawings=new CPNDrawing[netenum.numberOfRemainingElements()];
      for (int i=0; netenum.hasMoreElements(); ++i) {
	  ShadowNet net=(ShadowNet)netenum.nextElement();
	  drawings[i] = new ShadowNetRenderer(net).drawing;
      }
      return drawings;
  }


}

class ShadowNetRenderer {

  CPNDrawing drawing=new CPNDrawing();
  private Hashtable lookup=new Hashtable();
  private Point loc=new Point(100,20);

  ShadowNetRenderer(ShadowNet net) {
      render(net);
  }

  private void render(ShadowNet net)  {
      drawing.setName(net.getName());
      Enumeration elemenum=net.elements();
      while (elemenum.hasMoreElements()) {
	  Object element=elemenum.nextElement();
	  if (element instanceof ShadowNode) {
	      renderNode((ShadowNode)element);
	  }
	  if (element instanceof ShadowDeclarationNode) {
	      renderDeclarationNode((ShadowDeclarationNode)element);
	  }
      }
      elemenum=net.elements();
      while (elemenum.hasMoreElements()) {
	  Object element=elemenum.nextElement();
	  if (element instanceof ShadowArc) {
	      renderArc((ShadowArc)element);
	  }
      }
      elemenum=net.elements();
      while (elemenum.hasMoreElements()) {
	  Object element=elemenum.nextElement();
	  if (element instanceof ShadowInscription) {
	      renderInscription((ShadowInscription)element);
	  }
      }
  }

  private void renderDeclarationNode(ShadowDeclarationNode declNode) {
      DeclarationFigure createdFig=new DeclarationFigure();
      createdFig.setText(declNode.inscr);
      drawing.add(createdFig);
  }

  private void renderNode(ShadowNode node) {
      Dimension dim=null;
      ParentFigure createdFig=null;
      if (node instanceof ShadowTransition) {
	  createdFig = new TransitionFigure();
	  dim=((TransitionFigure)createdFig).defaultDimension();
      } else {
	  createdFig = new PlaceFigure();
	  dim=((PlaceFigure)createdFig).defaultDimension();
      }
      drawing.add(createdFig);
      if (node.getName()!=null) {
	  CPNTextFigure nameFig=new CPNTextFigure(CPNTextFigure.NAME);
	  nameFig.setText(node.getName());
	  int nameWidth=nameFig.displayBox().width;
	  if (nameWidth+8>dim.width)
	      dim.width=nameWidth+8;
	  drawing.add(nameFig);
	  nameFig.setParent(createdFig);
      }
      Point rightLower=new Point(dim.width,dim.height);
      createdFig.displayBox(new Point(),rightLower);
      createdFig.moveBy(loc.x,loc.y);
      //loc.y += 50;
      lookup.put(node,createdFig);
      createdFig.setAttribute("TraceMode",new Boolean(node.getTrace()));
  }

  private void renderArc(ShadowArc arc) {
      int shadowArcType=arc.shadowArcType;

      Figure startFig=(Figure)lookup.get(arc.place);
      Figure endFig=(Figure)lookup.get(arc.transition);
      if (!arc.placeToTransition) {
	Figure helpFig=startFig;
	startFig=endFig;
	endFig=helpFig;
      }

      ArcConnection arcFig = new ArcConnection(shadowArcType);
      arcFig.startPoint(0,0);
      arcFig.endPoint(0,0);
      drawing.add(arcFig);
      lookup.put(arc,arcFig);
      arcFig.connectStart(startFig.connectorAt(0,0));
      arcFig.connectEnd(endFig.connectorAt(0,0));
      arcFig.updateConnection();
      arcFig.setAttribute("TraceMode",new Boolean(arc.getTrace()));
  }

  private void renderInscription(ShadowInscription inscription) {
      CPNTextFigure inscrFig=new CPNTextFigure(CPNTextFigure.INSCRIPTION);
      inscrFig.setText(inscription.inscr);
      ParentFigure parent=(ParentFigure)lookup.get(inscription.inscribable);
      drawing.add(inscrFig);
      inscrFig.setParent(parent);
      inscrFig.setAttribute("TraceMode",new Boolean(inscription.getTrace()));
      if (parent instanceof ArcConnection) {
	  inscrFig.setAttribute("FillColor",CH.ifa.draw.util.ColorMap.BACKGROUND);
      }
  }

}

