package de.renew.gui;

import java.awt.*;
import java.util.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.util.*;
import de.renew.simulator.*;
import collections.*;

abstract public class SimpleCompositeFigure extends CompositeFigure {

    protected Rectangle fDisplayBox = new Rectangle(0,0,0,0);
    private Vector fHandles=null; // cache for handles of all included figures

    protected Vector fClickHandles=new Vector(); // additional ClickHandles

   protected void addClickHandle(ClickHandle handle) {
       fClickHandles.addElement(handle);
       fHandles=null;
       handlesChanged();
   }


/*
    public void draw(Graphics g) {
       g.setColor(Color.red);
       g.drawRect(fDisplayBox.x,fDisplayBox.y,fDisplayBox.width,fDisplayBox.height);
       super.draw(g);
    }
*/

    protected void basicMoveBy(int x, int y) {
	    fDisplayBox.translate(x, y);
	    super.basicMoveBy(x, y);
	}

    public void moveBy(int x, int y) {
        willChange();
        basicMoveBy(x, y);
        changed();
    }

    public Rectangle displayBox() {
        return new Rectangle(
            fDisplayBox.x,
            fDisplayBox.y,
            fDisplayBox.width,
            fDisplayBox.height);
    }

    public void basicDisplayBox(Point origin, Point corner) {
        fDisplayBox = new Rectangle(origin);
        fDisplayBox.add(corner);
        layout();
    }

    protected Vector basicHandles() {
	return new Vector();
    }

    public Vector handles() {
	if (fHandles==null) {
	  fHandles = basicHandles();
	  FigureEnumeration k = figures();
	  while (k.hasMoreElements()) {
	    Figure figure = k.nextFigure();
	    Enumeration kk = figure.handles().elements();
	    while (kk.hasMoreElements()) {
	      Object handle=kk.nextElement();
	      if (handle instanceof TokenHandle)
		fHandles.addElement(handle);
	    }
	  }
	  Enumeration clickHandles=fClickHandles.elements();
	  while (clickHandles.hasMoreElements())
	      fHandles.addElement(clickHandles.nextElement());
	}
	return fHandles;
    }

    protected void figureSetChanged() {
        super.figureSetChanged();
	fHandles=null;
	handlesChanged();
	//System.out.println("Resetting handles of "+this.getClass());
    }

    protected void layout() {
      // update ClickHandles:
      for (int i=0; i<fClickHandles.size(); ++i) {
	  ClickHandle handle=(ClickHandle)fClickHandles.elementAt(i);
	  Dimension size=handle.owner().size();
	  handle.setBox(new Rectangle(0,0,size.width,size.height));
      }
    }

    abstract protected boolean needsLayout();

    public void update(FigureChangeEvent e) {
      if (needsLayout()) {
	willChange();
	layout();
	changed();
      }
    }

    public void figureChanged(FigureChangeEvent e) {
        update(e);
    }

    public void figureHandlesChanged(FigureChangeEvent e) {
        fHandles=null;
	handlesChanged();
    }

    public void figureRemoved(FigureChangeEvent e) {
        update(e);
    }

    public boolean inspect(DrawingView view, boolean alternate) {
       Point click=view.lastClick();
       Figure inside = findFigureInside(click.x, click.y);
       if (inside != null && inside.inspect(view, alternate))
          return true;
        if (alternate) {
           return false; // don't use default alternate behaviour
        } else {
           return super.inspect(view,false);  // do use default normal behaviour
        }
    }

     public void setAttribute(String name, Object value) {
        Enumeration figenum = figures();
        while (figenum.hasMoreElements()) {
            ((Figure)figenum.nextElement()).setAttribute(name,value);
        }
     }

     public Object getAttribute(String name) {
        Enumeration figenum = figures();
        if (figenum.hasMoreElements()) {
            return ((Figure)figenum.nextElement()).getAttribute(name);
        }
        return null;
     }

}

