package de.renew.gui;

import java.awt.*;
import java.util.*;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import CH.ifa.draw.figures.*;
import CH.ifa.draw.util.*;
import de.renew.simulator.*;
import de.renew.event.*;
import collections.*;

public class TokenBagFigure extends SimpleCompositeFigure
    implements ChildFigure, PlaceEventListener {

    public TokenBagFigure() {
        super();
    }

    private static final int BORDER = 3;
    private static final int SEPARATOR = 1;
    private OffsetLocator fLocator = null;
    private RenewMode mode;
    private PlaceInstanceFigure place;
    private PlaceInstance placeInstance;
    private int cacheMult = -1;
    private int cacheTestMult = -1;
    private TextFigure cardFigure;
    private FigureAttributes attributes;
    private int markingAppearance=-1; // force first-time update

    public TokenBagFigure(RenewMode mode,
			  PlaceInstance pi) {
	this(mode,null,pi);
    }

    public TokenBagFigure(RenewMode mode,
			  PlaceInstanceFigure place,
				  PlaceInstance pi) {
       this(mode,place,pi,PlaceFigure.CARDINALITY);
    }

    public TokenBagFigure(RenewMode mode,
			  PlaceInstanceFigure place,
			  PlaceInstance pi,
			  int markingAppearance) {
       placeInstance = pi;
       this.mode = mode;
       cardFigure = new TextFigure();
       cardFigure.setReadOnly(true);
       cardFigure.setAttribute("FontStyle",new Integer(Font.ITALIC));
       attributes = new FigureAttributes();
       if (place!=null)
	   setParent(place);
       setMarkingAppearance(markingAppearance);
       placeInstance.addPlaceEventListener(this);
    }

    protected Vector basicHandles() {
	//    public Vector handles() {
	Vector handles=super.basicHandles();
	handles.addElement(new NullHandle(this, RelativeLocator.northWest()));
	handles.addElement(new NullHandle(this, RelativeLocator.northEast()));
	handles.addElement(new NullHandle(this, RelativeLocator.southWest()));
	handles.addElement(new NullHandle(this, RelativeLocator.southEast()));
	return handles;
    }

    /*
    public void draw(Graphics g) {
       if (expanded) {
         g.setColor(Color.red);
         Rectangle box=displayBox();
         g.drawRect(box.x,box.y,box.width-1,box.height-1);
       }
       super.draw(g);
    }
    */

    public boolean canBeParent(ParentFigure p) {
       return p instanceof PlaceInstanceFigure;
    }

    private void unsetParent() {
      if (place != null) {
	place.removeChild(this);
	place=null;
      }
    }

    public boolean setParent(ParentFigure figure) {
        if (!canBeParent(figure))
	  return false;
	if (place == figure)
	  return true;
	unsetParent();
        place = (PlaceInstanceFigure)figure;
	if (place == null) {
          fLocator = null;
        } else {
	  place.addChild(this);
          if (fLocator != null)
             fLocator.setBase(place.connectedTextLocator(this));
          else
	     fLocator = new OffsetLocator(place.connectedTextLocator(this));
	  updateLocation();
	}
	return true;
    }

    public ParentFigure parent() { return place; }

    protected void layout() {
       Point partOrigin = displayBox().getLocation();
       partOrigin.translate(BORDER,BORDER);
       Dimension extent = new Dimension(0, 0);

       FigureEnumeration k = figures();
       while (k.hasMoreElements()) {
            Figure f = k.nextFigure();

 	    Dimension partExtent = f.size();
	    Point corner = new Point(
		                        partOrigin.x+partExtent.width,
		                        partOrigin.y+partExtent.height);
            f.basicDisplayBox(partOrigin, corner);

	    extent.width = Math.max(extent.width, partExtent.width);
	    extent.height += partExtent.height+SEPARATOR;
	    partOrigin.y += partExtent.height+SEPARATOR;
        }
        fDisplayBox.width = extent.width + 2*BORDER;
        fDisplayBox.height = extent.height - SEPARATOR + 2*BORDER;
        updateLocation();
	super.layout();
    }

    protected boolean needsLayout() {
        Dimension extent = new Dimension(0, -SEPARATOR + 2*BORDER);

        FigureEnumeration k = figures();
        while (k.hasMoreElements()) {
            Figure f = k.nextFigure();
            extent.width = Math.max(extent.width, f.size().width);
            extent.height += f.size().height + SEPARATOR;
        }
        extent.width += 2*BORDER;
        return !extent.equals(fDisplayBox.getSize());
    }

    public void moveBy(int x, int y) {
        super.moveBy(x, y);
        if (fLocator != null)
            fLocator.moveBy(x, y);
    }

    /**
     * Updates the location relative to the connected figure.
     * The Figure is centered around the located point.
     */
    public void updateLocation() {
        if (fLocator != null) {
            Point p = fLocator.locate(place);
            Rectangle box=displayBox();
            p.x -= box.x + box.width/2;
            p.y -= box.y + box.height/2;

            if (p.x != 0 || p.y != 0) {
                willChange();
                basicMoveBy(p.x, p.y);
                changed();
            }
        }
    }


  /** This event method is called whenever the marking of
   *  the corresponding PlaceInstance changes dramatically
   *  (i.e. by more than one token).
   */
  public void markingChanged(PlaceEvent pe) {
    // Updates only originate from place instances.
    if (pe.getPlaceInstance() != placeInstance) {
        // This is strange: Why should the attached place instance
	// change?
	System.out.println("Wrong PlaceEvent received!");
	throw new RuntimeException();
	// Other solution: make placeInstance flexible (not tested)
	//      placeInstance.removePlaceEventListener(this);
	//      placeInstance = (PlaceInstance)oPlaceInstance;
	// construct new cardFigure!
    }

    if (markingAppearance==PlaceFigure.CARDINALITY) {
      updateCardFigure();
    } else {
      placeInstance.lock.lock();
      removeAll();
      Enumeration tokens = placeInstance.getDistinctTestableTokens();

      willChange();

      double currentTime=SearchQueue.getTime();
      boolean expanded=markingAppearance==PlaceFigure.EXPANDED_TOKENS;
      while (tokens.hasMoreElements()) {
	Object token=tokens.nextElement();
	
	TimeSet timeSet=placeInstance.getFreeTimeSet(token);
	boolean isTested=placeInstance.containsTestedToken(token);
	TimeSet extendedTimeSet;
	if (isTested) {
	  extendedTimeSet=timeSet.including(currentTime);
	} else {
	  extendedTimeSet=timeSet;
	}
	double[] times=extendedTimeSet.asUniqueArray();
	
	for (int i=0;i<times.length;i++) {
	  double time=times[i];
	  int mult=timeSet.multiplicity(time);
	  boolean showTested=isTested && time==currentTime;
	  Figure tokenFigure;
//  	  if (mult==1 && !showTested && time==0) {
//  	      tokenFigure=mode.getTokenFigure(token,markingAppearance==EXPANDED_TOKENS);
//  	      if (token instanceof NetInstance) {
//  		  addClickHandle(new NetInstanceHandle(tokenFigure,null,
//  						       (NetInstance)token));
//  	      }
//  	  } else
	      tokenFigure=new MultipleTokenFigure(mode,mult,showTested,time,
						  token,expanded);
	  Enumeration attrenum=attributes.definedAttributes();
	  while (attrenum.hasMoreElements()) {
	    String attribute=(String)attrenum.nextElement();
	    tokenFigure.setAttribute(attribute,attributes.get(attribute));
	  }
	  add(tokenFigure);
	}
      }
      
      placeInstance.lock.unlock();

      layout();
      changed();

      if (listener() != null)
	listener().figureRequestUpdate(new FigureChangeEvent(this));
    }

  }

  /** This event occurs when a single token (contained in the
   *  event object) is put into the corresponding PlaceInstance.
   */
  public void tokenAdded(TokenEvent te) {
      markingChanged(new PlaceEvent(te.getPlaceInstance()));
  }

  /** This event occurs when a single token (contained in the
   *  event object) is removed from the corresponding PlaceInstance.
   */
  public void tokenRemoved(TokenEvent te) {
      tokenAdded(te);
  }

  /** This event occurs when a token (contained in the
   *  event object) is tested within the corresponding PlaceInstance.
   */
  public void tokenTested(TokenEvent te) {
      tokenAdded(te);
  }

  /** This event occurs when a token (contained in the
   *  event object) is untested within the corresponding PlaceInstance.
   */
  public void tokenUntested(TokenEvent te) {
      tokenAdded(te);
  }


  private void updateCardFigure() {
    placeInstance.lock.lock();
    int newMult = placeInstance.getNumberOfTokens();
    int newTestMult = placeInstance.getNumberOfTestedTokens();
    placeInstance.lock.unlock();

    if ((newMult!=cacheMult) || (newTestMult!=cacheTestMult)) {
       cacheMult = newMult;
       cacheTestMult = newTestMult;

       if (newTestMult > 0) {
	 if (newMult>0) {
           cardFigure.setText(newMult+"("+newTestMult+")");
	 } else {
	   cardFigure.setText("("+newTestMult+")");
	 }
       } else if (newMult > 0)
	 cardFigure.setText(String.valueOf(newMult));
       else
	 cardFigure.setText(" ");

       if (listener() != null)
          listener().figureRequestUpdate(new FigureChangeEvent(this));

    }
  }


  public void release() {
    if (placeInstance != null) {
      placeInstance.removePlaceEventListener(this);
      // OK: Release reference to place instance so that the place
      // instance can be garbage collected.
      placeInstance=null;
    }
    // OK: Forget about the place.
    unsetParent();
    // OK: Forget about all tokens.
    removeAll();

    super.release();
  }

  public String toString() {
    if (placeInstance==null) {
      return "Detached TokenBagFigure@"+System.identityHashCode(this);
    }

    StringBuffer output = new StringBuffer().append('{');

    placeInstance.lock.lock();

    Enumeration tokens = placeInstance.getDistinctTestableTokens();
    while (tokens.hasMoreElements()) {
      Object token=tokens.nextElement();
      int mult=placeInstance.getTokenCount(token);
      boolean isTested=placeInstance.containsTestedToken(token);

      output.append(MultipleTokenFigure.getMultString(mult,isTested));
      output.append(token.toString());

      if (tokens.hasMoreElements())
	output.append(", ");
    }

    placeInstance.lock.unlock();

    output.append('}');
    return output.toString();
  }

  public void setMarkingAppearance(int markingAppearance) {
    if (this.markingAppearance != markingAppearance
	&& markingAppearance != PlaceFigure.HIGHLIGHT) {
	willChange();
	this.markingAppearance = markingAppearance;
	if (markingAppearance==PlaceFigure.CARDINALITY) {
	    removeAll();
	    add(cardFigure);
	}
	markingChanged(new PlaceEvent(placeInstance));
	layout();
	changed();
    }
  }

  /** Returns whether this figure can be selected.
   */
    //  public boolean isSelectable() { return false; }

  public boolean inspect(DrawingView view, boolean alternate) {
     boolean done=super.inspect(view,alternate);
     if (alternate && !done) {
	 view.clearSelection();
	 int newMA;
	 if (markingAppearance==PlaceFigure.CARDINALITY) {
	     if (place==null)
		 // in seperate window:
		 newMA=PlaceFigure.EXPANDED_TOKENS;
	     else
		 newMA=PlaceFigure.TOKENS;
	 } else {
	     newMA=PlaceFigure.CARDINALITY;
	 }
	 setAttribute("MarkingAppearance", new Integer(newMA));
	 view.addToSelection(this);
     }
     return done;
  }

  protected PlaceInstance getPlaceInstance() {
     return placeInstance;
  }


  public void update(FigureChangeEvent e) {
      updateLocation();
      super.update(e);
  }

  public void setAttribute(String name, Object value) {
      if (name.equals("MarkingAppearance")) {
	  if (place != null) {
	      place.setAttribute(name,value);
	  }
	  // if we want TokenBagFigures to also
	  // be switchable if there is no PlaceInstanceFigure
	  // assigned, we need:
	  else {
	     setMarkingAppearance(((Integer)value).intValue());
	  }
      } else {
	  if (markingAppearance!=PlaceFigure.CARDINALITY) {
	      attributes.set(name,value);
	  }
	  super.setAttribute(name,value);
      }
  }

}

