package de.renew.gui;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import collections.CollectionEnumeration;
import CH.ifa.draw.framework.*;
import CH.ifa.draw.standard.*;
import de.renew.simulator.*;
import de.renew.event.*;
import de.renew.unify.Variable;

class TransitionInstanceFigure extends AbstractFigure
    implements InstanceFigure,
	       TransitionEventListener {

  private TransitionFigure transitionFigure;
  private TransitionInstance transitionInstance;
  private int actionCounter=0;
  private boolean afterglow=false;
  private CPNInstanceDrawing drawing;

  public TransitionInstanceFigure(CPNInstanceDrawing drawing,
				  TransitionFigure tf, TransitionInstance ti) {
	this.drawing = drawing;

        transitionInstance = ti;
        transitionFigure = tf;

	synchronized (transitionInstance) {
	  CollectionEnumeration enum=transitionInstance.pendingFiringEvents();
	  while (enum.hasMoreElements()) {
	    FiringEvent fe=(FiringEvent)enum.nextElement();
	    firingStarted(fe);
	  }

	  transitionInstance.addTransitionEventListener(this);
	}
  }

  synchronized public boolean isHighlighted() {
      return actionCounter > 0 || afterglow;
  }

  private void invalidateHighlight() {
      Figure highlight=transitionFigure.getHighlightFigure();
      if (highlight!=null) {
	  Rectangle area = highlight.displayBox();
	  area.grow(5,5);
	  DrawingChangeEvent dce = new DrawingChangeEvent(drawing,area);
	  drawing.drawingInvalidated(dce);
      }
  }


    /** This event occurs when a TransitionInstance starts a new
     *  Occurrence. When the event is sent, it is guaranteed
     *  that the firing can take place, i.e., all input tokens have 
     *  already been successfully collected.
     *
     *  The Event is sent to all TransitionInstances which are part of the
     *  Occurrence, not only to the triggering TransitionInstance.
     */
    public void firingStarted(FiringEvent fe) {
	//System.out.println("Firing "+transitionInstance);
	synchronized(this) {
	    ++actionCounter;
	    if (actionCounter == 1 && listener() != null) {
		invalidate();
		invalidateHighlight();
		listener().figureRequestUpdate(new FigureChangeEvent(this));
	    }
	}
    }

    /** This event signals that a Transition Occurrence has completed.
     *  The Event is sent to all TransitionInstances which are part of the
     *  Occurrence, not only to the triggering TransitionInstance.
     *
     *  The event that is delivered is exactly the same event that was
     *  delivered during the start of the firing.
     */
    public void firingComplete(FiringEvent fe) {
	//System.out.println("Firing of "+transitionInstance+" completed.");
	synchronized(this) {
	    --actionCounter;
	    if (actionCounter == 0 && listener() != null) {
		afterglow=true;
		final TransitionInstanceFigure thizz=this;
		new Thread() {
		    public void run() {
			try { sleep(100); }
			catch (InterruptedException e) { }
			synchronized(thizz) {
			    afterglow=false;
			}
			invalidate();
			invalidateHighlight();
			listener().figureRequestUpdate(new FigureChangeEvent(thizz));
		    }
		}.start();
	    }
	}
    }

    public void basicDisplayBox(Point origin, Point corner) {
        // do nothing, as the DisplayBox of the Transition is used!
    }

    public Vector handles() {
        Vector handles = new Vector();
        handles.addElement(new NullHandle(this, RelativeLocator.northWest()));
        handles.addElement(new NullHandle(this, RelativeLocator.northEast()));
        handles.addElement(new NullHandle(this, RelativeLocator.southWest()));
        handles.addElement(new NullHandle(this, RelativeLocator.southEast()));
        return handles;
    }

    public Rectangle displayBox() {
        return transitionFigure.displayBox();
    }

    protected void basicMoveBy(int x, int y) {
        // do nothing, as the DisplayBox of the Transition is used!
    }

    public void draw(Graphics g) {
	/*	if (actionCounter > 0) {
	    g.setColor(Color.red);
	    Rectangle r = displayBox();
	    g.fillRect(r.x, r.y, r.width, r.height);
	}
	*/
    }


  public boolean inspect(DrawingView view, boolean alternate) {
    //System.out.println("Searching bindings for "+transitionInstance+"...");
    if (alternate) {
      if (fire()) {
	view.editor().showStatus("Transition "+transitionInstance+" fired.");
      } else {
	view.editor().showStatus("No binding for this transition.");
      }
    } else {
      BindingSelectionFrame.open(transitionInstance,getSimulation());
    }
    return true;
  }

  private CPNSimulation getSimulation() {
    return drawing.getMode().getSimulation();
  }

  public boolean fire() {
    CPNSimulation simulation=getSimulation();
    simulation.getBreakpointManager().clearLog();
    boolean result=SimulatorHelper.fireOneBinding(transitionInstance,
      new ConsoleTracer("Synchronously","(Automatically) "), 
      !simulation.isStrictlySequential());
    simulation.simulationRefresh();
    return result;
  }

  public void release() {
    if (transitionInstance != null) {
	transitionInstance.removeTransitionEventListener(this);
	// OK: Release reference to transition instance so that it
	// can be garbage collected.
	transitionInstance=null;
    }
    super.release();
  }

  protected TransitionInstance getInstance() {
    return transitionInstance;
  }
}
