package de.renew.gui.fs;

import CH.ifa.draw.figures.*;
import java.awt.*;
import collections.*;
import java.util.*;
import java.io.*;
import de.renew.formalism.fs.*;
import de.renew.gui.*;
import de.uni_hamburg.fs.*;
import de.renew.simulator.NetInstance;
import de.renew.formalism.java.JavaNetCompiler;
import de.renew.formalism.java.ParsedDeclarationNode;
import de.renew.shadow.SyntaxException;
import CH.ifa.draw.util.*;
import CH.ifa.draw.framework.DrawingView;
import CH.ifa.draw.standard.RelativeLocator;

public class FSFigure extends CPNTextFigure
                      implements SemanticUpdateFigure {

  public static final int SHUTTERSIZE=7;
  public static final String ELLIPSE="...";

  transient private FeatureStructure fs;
  transient private TagMap tagmap;
  transient private FontMetrics metrics, boldMetrics;
  private int lineh, d, ascent;
  private Dimension fExtent=null;
  private UpdatableSeq subfigs=null, // of Drawables;
                       boldfigs=null; //of TextSubFigures 
  private UpdatableSet openNodes=new HashedSet();
    // of Nodes which are openNodes
  private Node selectedNode=null;
  private UpdatableSeq handles=null; // of Tag- and ShutterHandles
  private int updateHandleIndex=-1; // if we only update handle rects,
                                    // this variable contains the current
                                    // index within handles, -1 otherwise.

  public static boolean umlMode=true;

  public FSFigure(Object token) {
    this(new FeatureStructure(JavaObject.getJavaType(token)),false);
  }

  public FSFigure(FeatureStructure fs) {
      this(fs,false);
  }

  public FSFigure(FeatureStructure fs, boolean expanded) {
    super(INSCRIPTION);
    setFrameColor(Color.black); // so that TextFigure treats us as a box
    this.fs = fs;
    tagmap = new TagMap(fs.getRoot());
    setReadOnly(true);
    super.setText(" "); // non-empty, so that this figure is not removed!
    if (expanded) {
	// initially all nodes are open:
	openNodes.includeElements(fs.getNodes());
	// for JavaObjects, open first level:
	openNodes.include(fs.getRoot());
    }
  }

  public FSFigure() {
    super(INSCRIPTION);
    setFrameColor(Color.black); // so that TextFigure treats us as a box
    setText("[]");
    setReadOnly(false);
  }

  protected void basicSetText(String newText)  {
    super.basicSetText(newText);
    if (!readOnly()) {
      try {
	semanticUpdate(null);
      } catch (SyntaxException se) {
	fs=null;
	System.out.println("Syntax Exception in Feature Structure:\n"+se);
      }
    }
  }

  FeatureStructure getFeatureStructure() {
    return fs;
  }

  /** This method is called by the CPNTextTool after endEdit() and
   * setText() and gives SemanticUpdateFigures the opportunity to update
   * their semantics with the information of the Declaration Node.
   */
  public void semanticUpdate(ParsedDeclarationNode declNode)
              throws SyntaxException {
    synchronized(this) {
      updateHandleIndex=-1;
      FSNetParser fsParser=new FSNetParser(new java.io.StringReader(getText()));
      fsParser.setDeclarationNode(declNode);
      UpdatableMap tags=new HashedMap();
      EquivRelation er = new EquivRelation();
      selectedNode=null;
      Node root=null;
      try {
	root = fsParser.parseFS(tags,er,Path.EPSILON,new Vector(),new Vector());
      } catch (de.renew.formalism.java.ParseException ex) {
	SyntaxException e=JavaNetCompiler.makeSyntaxException(ex);
	e.addObject(this);
	changed();
	throw e;
      }
      try {
	er.extensionalize();
	root=er.rebuild(root);
      } catch (UnificationFailure uff) {
	SyntaxException e=new SyntaxException("FS not extensionalizable!",null);
	e.addObject(this);
	changed();
	throw e;
      }
      fs=new FeatureStructure(root,false);
      tagmap = new TagMap(root,er,tags);
      openNodes=new HashedSet();
      // initially all fs-nodes are open:
      openNodes.includeElements(fs.getNodes());
      // for JavaObjects, open first level:
      openNodes.include(root);
      changed();
      //System.out.println("FS: "+new FeatureStructure(fs));
    }
  }


  public Vector handles() {
    Vector allHandles=super.handles();
    synchronized(this) {
      if (handles!=null) {
	Enumeration handleEnum=handles.elements();
	while (handleEnum.hasMoreElements()) {
	    allHandles.addElement(handleEnum.nextElement());
	}
      }
    }
    return allHandles;
  }

  public static Font getBoldFont(Font font) {
      return Fontkit.getFont(font.getName(), font.getStyle() | Font.BOLD,
			     font.getSize());
  }

  private void initMetrics() {
      metrics = Toolkit.getDefaultToolkit().getFontMetrics(getFont());
      boldMetrics = Toolkit.getDefaultToolkit().getFontMetrics(getBoldFont(getFont()));
      lineh =  metrics.getHeight();
      d = metrics.stringWidth(" ");
      ascent = metrics.getAscent()-1;
  }

  public void setup() {
    if (fExtent == null) {
      synchronized(this) {
        tagmap.resetVisited();
        initMetrics();
	subfigs=new LinkedList();
	boldfigs=new LinkedList();
	if (updateHandleIndex<0)
	    handles=new LinkedList();
        fExtent = setupFS(0,0,fs.getRoot(),Type.TOP);
	updateHandleIndex=0;
      }
    }
  }

  public Rectangle displayBox() {
    Rectangle box = super.displayBox();
    if (fs==null)
	return box;
    setup();
    return new Rectangle(box.x, box.y, fExtent.width, fExtent.height);
  }

  public void draw(Graphics g) {
    if (isVisible()) {
	if (fs == null) {
	    super.draw(g);
	} else {
	  synchronized(this) {
	    setup();
	    Rectangle box = super.displayBox();
	    Color fill = getFillColor();
	    if (!ColorMap.isTransparent(fill)) {
		g.setColor(fill);
		g.fillRect(box.x,box.y,fExtent.width,fExtent.height);
	    }
	    g.setColor((Color)getAttribute("TextColor"));
	    g.setFont(getFont());
	    g.translate(box.x,box.y);
	    Enumeration subfigenum=subfigs.elements();
	    while (subfigenum.hasMoreElements()) {
		((Drawable)subfigenum.nextElement()).draw(g);
	    }
	    g.setFont(getBoldFont(getFont()));
	    subfigenum=boldfigs.elements();
	    while (subfigenum.hasMoreElements()) {
		((Drawable)subfigenum.nextElement()).draw(g);
	    }
	    g.translate(-box.x,-box.y);
	  }
	}
    }
  }

  private boolean addShutter(Node fs, int x, int y) {
    boolean isClosed=!openNodes.includes(fs);
    Rectangle shutterBox=new Rectangle(x-SHUTTERSIZE/2,y-SHUTTERSIZE/2,
				       SHUTTERSIZE,SHUTTERSIZE);
    if (updateHandleIndex<0) {
      handles.insertLast(new ShutterHandle(this,shutterBox,fs,isClosed));
    } else {
      ShutterHandle sh=(ShutterHandle)handles.at(updateHandleIndex++);
      if (sh.node!=fs)
	System.err.println("test failed: TagHandle references "+sh.node+" instead of "+fs);
      sh.setBox(shutterBox);
      sh.isClosed=isClosed;
    }
    return isClosed;
  }

  private Dimension setupList(int x, int y, int maxh,
			      Node fs) {

    int w = 0;
    Type listtype=fs.getType();
    try {
    ListType list=(ListType)listtype;
    Type elemtype=list.getBaseType();

      if (list.getSubtype()==ListType.NELIST) {
	if (openNodes.includes(fs)) {
	  // list node is not closed:
	  Node head;
	  if (fs.hasFeature(ListType.HEAD)) {
	    head = fs.delta(ListType.HEAD);
	  } else {
	    head = new FSNode(elemtype);
	  }
	  Dimension headdim = setupFS(x,y,head,elemtype);
	  maxh = Math.max(maxh,headdim.height);
	  w=headdim.width;

	  Node tail;
	  if (fs.hasFeature(ListType.TAIL)) {
	    tail = fs.delta(ListType.TAIL);
	  } else {
	    tail = new FSNode(ListType.getList(elemtype));
	  }
	  Dimension taildim=null;
	  Name tag = tagmap.getTag(tail);
	  listtype=tail.getType();
	  if (listtype instanceof ListType) {
	    ListType tailtype=(ListType)listtype;
	    if (tailtype.getSubtype()==ListType.NELIST &&
		tag.equals(de.uni_hamburg.fs.Name.EMPTY) &&
		tailtype.getBaseType().equals(elemtype)
		||
		tailtype.getSubtype()==ListType.ELIST) {
	      taildim = setupList(x+w+2*d,y,maxh,tail);
	      if (taildim.width>d)
		  w+=2*d;
	  }
	  if (taildim==null) {
	    taildim = setupFS(x+w + 3*d-1,y,tail,ListType.getList(elemtype));
	    // add list seperator line:
	    subfigs.insertFirst(new LineSubFigure(x+w+3*d/2,y,0,maxh));
	    w += 3*d; // space for vertical line
	  }
	  w += taildim.width;
	  maxh = Math.max(maxh,taildim.height);
	}
      }
    }
    } catch (ClassCastException cce) {
	System.err.println("!!!corrupted list! Unexpected type: "+listtype);
    }
    if (maxh==0)
	maxh=lineh;
    if (w==0)
	w=d;
    return new Dimension(w,maxh);
  }

  private Dimension setupFS(int x, int y, Node fs,
			    Type type) {
if (fs==null) return new Dimension(0,0);
    int maxh = lineh, maxw = 0;
    Type nodetype = fs.getType();
    int left = x, upper=y;
    boolean isNode=!(nodetype instanceof BasicType
		     && ((BasicType)nodetype).isObject()
		     || nodetype instanceof NullObject);
    boolean isList=false;
    if (nodetype instanceof ListType) {
      int subtype=((ListType)nodetype).getSubtype();
      if  ((subtype==ListType.ELIST ||
	    subtype==ListType.NELIST &&
	    fs.hasFeature(ListType.HEAD) &&
	    fs.hasFeature(ListType.TAIL))
	   && !tagmap.isVisited(fs))
	isList=true;
    }
    if (isNode) {
	left+=d; // space for square bracket
	if (isList) {
	  left+=d; // additional space for angles
	  // convert types to base types:
	  if (type instanceof ListType) {
	    type=((ListType)type).getBaseType();
	  }
	  nodetype=((ListType)nodetype).getBaseType();
	} else {
	  upper+=2;
	}
    }
    boolean noFirstLine=true;
    Rectangle tagbox=null;
    Name tag = tagmap.getTag(fs);
    if (!tag.equals(de.uni_hamburg.fs.Name.EMPTY)) {
	// tags are only drawn for taged nodes, not for basic types or
	// nodes without features of an extensional type.
	String tagstr = tag.name;
	if (nodetype instanceof JavaObject &&
	    ((JavaObject)nodetype).getJavaObject() instanceof NetInstance)
	    tagstr=((JavaObject)nodetype).getJavaObject().toString();
	int tagwidth;
	if (umlMode) {
	  tagwidth = boldMetrics.stringWidth(tagstr);
	  if (nodetype.isInstanceType()) {
	    maxh+=1; // for underline
	    Drawable tagUL=new LineSubFigure(left,upper+ascent+1,tagwidth,0);
	    subfigs.insertFirst(tagUL);
	  }
	  boldfigs.insertFirst(new TextSubFigure(tagstr,left,upper+ascent));
	} else {
	  tagwidth = metrics.stringWidth(tagstr)+d;
	  subfigs.insertFirst(new TextSubFigure(tagstr,left+(d+1)/2,
						upper+ascent));
	  Drawable tagBX=new FilledRectSubFigure(left,upper,tagwidth,lineh-2);
	  subfigs.insertFirst(tagBX);
	}
	tagbox=new Rectangle(left,upper,tagwidth,lineh-2);
	maxw = tagwidth;
	noFirstLine=false;
    }
    if (!isNode || !tagmap.visit(fs)) {
      if (!isList && nodetype.getName().equals(FSNetCompiler.LINK)) {
	  // special rendering of FS-Up-/Downlinks:
	  int lx=0;
	  if (fs.hasFeature(FSNetCompiler.RCV)) {
	      Dimension rcvdim = setupFS(x,y,fs.delta(FSNetCompiler.RCV),
					 nodetype.appropType(FSNetCompiler.RCV));
	      maxh = rcvdim.height;
	      lx = rcvdim.width + d;
	  }
	  Dimension subdim = setupFS(x+lx+2*d,y,fs.delta(FSNetCompiler.PARAM),
				     nodetype.appropType(FSNetCompiler.PARAM));
	  maxh = Math.max(maxh,subdim.height);
	  maxw = lx + subdim.width + 2*d;
	  subfigs.insertFirst(new TextSubFigure(":",x+lx, y+ascent));
	  return new Dimension(maxw,maxh);
      } else {
	    // draw optional first line with type (after tag)
	    String typestr = nodetype.getName();
	    if (!nodetype.equals(type) && typestr.length()>0) {
		//System.out.println("nodetype: "+nodetype+", default type: "+type);
	      if (umlMode && isNode)
		typestr=":"+typestr;
	      if (!umlMode && !tag.equals(de.uni_hamburg.fs.Name.EMPTY))
		  maxw+=d; // space between tag and type
	      Rectangle typeRect=
		  new Rectangle(left+maxw,upper,
				boldMetrics.stringWidth(typestr),lineh);
	      boldfigs.insertFirst(new TextSubFigure(typestr,
						     typeRect.x,upper+ascent));
	      if (isNode && nodetype.isInstanceType()) {
		subfigs.insertFirst(new LineSubFigure(left+maxw,upper+ascent+1,
						     typeRect.width,0));
		if (!umlMode)
		  maxh+=1; // space for underline
	      }
	      maxw+=typeRect.width;
	      noFirstLine=false;
	    }
	    Enumeration features=fs.featureNames();
	    boolean hasFeatures=features.hasMoreElements();
	    if (hasFeatures) {
	      if (!openNodes.includes(fs)) {
		subfigs.insertFirst(new TextSubFigure(ELLIPSE,left+maxw,
						     upper+ascent));
		maxw += metrics.stringWidth(ELLIPSE);
	      } else {
		if (noFirstLine)
		  maxh = 0;
		else if (umlMode && !noFirstLine)
		    maxh += 1; // space for separating line
		if (isList) {
		    Dimension listdim = setupList(left+1,upper+maxh,0,fs);
		    maxh += listdim.height;
		    maxw = Math.max(maxw,listdim.width);
		} else {
		  do {
		    Name featureName = (Name)features.nextElement();
		    String feature = featureName.toString()
			             + (umlMode ? "=" : ": ");
		    int indent = metrics.stringWidth(feature);
		    Dimension subdim = setupFS(left+indent,upper+maxh,
					       fs.delta(featureName),
					    nodetype.appropType(featureName));
		    subfigs.insertFirst(new TextSubFigure(feature, left,
				  upper+maxh+(subdim.height-lineh)/2+ascent));
		    maxw = Math.max(maxw,indent + subdim.width);
		    maxh += subdim.height + 1;  // + d;
		  } while (features.hasMoreElements());
		  maxh += 1;
		  if (umlMode && !noFirstLine) {
		    subfigs.insertFirst(new LineSubFigure(x,y+lineh+1,
							 maxw+2*d-1,0));
		  }
		}
	      }
	    }
	    if (hasFeatures) {
	      if (isList)
		addShutter(fs,x,y+maxh/2);
	      else
		addShutter(fs,x,y+lineh/2);
	    }
      }
    }
    if (maxw==0)
	maxw=d;
    if (isNode) {
	maxw+=2*d;
	if (isList) {
	    maxw+=2*d;
	} else {
	    maxh+=2;
	}
	Drawable tagsubfigure;
	if (umlMode && !isList) {
	    subfigs.insertFirst(new FilledRectSubFigure(x,y,maxw,maxh));
	    tagsubfigure=new RectSubFigure(x,y,maxw,maxh);
	} else {
	    tagsubfigure=new BracketSubFigure(x,y,maxw,maxh,
					      isList ? 2*d-1 : d-1,isList);
	    subfigs.insertFirst(tagsubfigure);
	}
	if (tagbox!=null) {
	  if (nodetype instanceof JavaObject &&
	      ((JavaObject)nodetype).getJavaObject() instanceof NetInstance) {
	    if (updateHandleIndex<0) {
	      handles.insertLast(new NetInstanceHandle
				 (this,tagbox,(NetInstance)
				  ((JavaObject)nodetype).getJavaObject()));
	    } else {
	      NetInstanceHandle nih=(NetInstanceHandle)handles.at(updateHandleIndex++);
	      nih.setBox(tagbox);
	    }
	  } else {
	    if (updateHandleIndex<0) {
		// create new tag handles:
	    } else {
		// update existing tag handles:
		TagHandle th=(TagHandle)handles.at(updateHandleIndex++);
		if (th.node!=fs)
		    System.err.println("test failed: TagHandle references "+th.node+" instead of "+fs);
		th.setBox(tagbox);
		th.setHighlight(tagsubfigure);
		th.selected=fs.equals(selectedNode);
	    }
	  }
	}
    }
    return new Dimension(maxw,maxh);
  }

  public void read(StorableInput dr) throws IOException {
    if (dr.getVersion() <= 5)
      // Hack for old FS-Figures:
      super.readWithoutType(dr);
    else {
      super.read(dr);
      setFrameColor(Color.black); // so that TextFigure treats us as a box
      if (dr.getVersion() > 6) {
	 // read list of paths for closed nodes:
	int noPaths=dr.readInt();
	Node root=fs.getRoot();
	while (noPaths-->0) {
	  Path path=new Path(dr.readString());
	  openNodes.include(root.delta(path));
	}
      }
    }
  }

  public void write(StorableOutput dw) {
    super.write(dw);
    if (fs==null) {
      dw.writeInt(0);
    } else {
      dw.writeInt(openNodes.size());
      Enumeration nodes=openNodes.elements();
      while (nodes.hasMoreElements()) {
	dw.writeString(fs.onePathTo((Node)nodes.nextElement()).toString());
      }
    }
  }

  protected void markDirty() {
    fExtent=null;
    super.markDirty();
  }

  private Enumeration getListNodes(Node node) {
    Vector listNodes=new Vector();
    listNodes.addElement(node);
    Type listType=node.getType();
    if (listType instanceof ListType &&
      ((ListType)listType).getSubtype()==ListType.NELIST) {
      while (true) {
	node=node.delta(ListType.TAIL);
	if (!node.getType().equals(listType) || listNodes.contains(node))
	    break;
	listNodes.addElement(node);
      }
    }
    return listNodes.elements();
  }


  void setNodeShutState(Node node, boolean close, boolean deep,
			DrawingView view) {
    synchronized(this) {
      Enumeration nodes;
      if (deep) {
	nodes=new FeatureStructure(node,false).getNodes();
      } else {
	nodes=getListNodes(node);
      }
      if (shutOrClose(nodes,close)) {
	handlesChanged();
	invalidate();
	markDirty();
	selectedNode=null;
	updateHandleIndex=-1;
	changed();
      }
    }
  }

  private boolean shutOrClose(Enumeration nodes, boolean close) {
    boolean changed=false;
    while (nodes.hasMoreElements()) {
      changed|=shutOrClose((Node)nodes.nextElement(),close);
    }
    return changed;
  }

  private boolean shutOrClose(Node node, boolean shut) {
    boolean isClosed=!openNodes.includes(node);
    if (isClosed==shut)
	return false;
    if (shut) {
      openNodes.removeOneOf(node);
    } else {
      openNodes.include(node);
    }
    return true;
  }

  void setSelectedTag(Node node, DrawingView view) {
    synchronized(this) {
      if (node.equals(selectedNode))
	// already selected, toggle to unselected
	selectedNode=null;
      else
	selectedNode=node;
      invalidate();
      Enumeration tags=handles.elements();
      while (tags.hasMoreElements()) {
	Object handle=tags.nextElement();
	if (handle instanceof TagHandle) {
	  TagHandle th=(TagHandle)handle;
	  th.selected=th.node.equals(selectedNode);
	}
      }
      if (selectedNode!=null) {
	// make all tagrects of this tag visible:
	if (shutOrClose(fs.backwardsReachableNodes(selectedNode),false)) {
	  handlesChanged();
	  markDirty();
	  updateHandleIndex=-1;
	}
      }
      changed();
    }
  }

}

abstract class PosSubFigure implements Drawable {
  int x, y;

  PosSubFigure(int x, int y) {
    this.x=x;
    this.y=y;
  }
}
  
class TextSubFigure extends PosSubFigure {
  String text;

  TextSubFigure(String text, int x, int y) {
    super(x,y);
    this.text=text;
  }

  public void draw(Graphics g) {
      g.drawString(text,x,y);
  }
}

class RectSubFigure extends PosSubFigure {
  int width, height;

  RectSubFigure(int x, int y, int width, int height) {
    super(x,y);
    this.width=width;
    this.height=height;
  }

  public void draw(Graphics g) {
    g.drawRect(x,y,width-1,height-1);
  }
}

class FilledRectSubFigure extends RectSubFigure {

  FilledRectSubFigure(int x, int y, int width, int height) {
    super(x,y,width,height);
  }

  public void draw(Graphics g) {
    Color penCol=g.getColor();
    g.setColor(Color.white);
    g.fillRect(x,y,width,height);
    g.setColor(penCol);
    super.draw(g);
  }
}

class LineSubFigure extends PosSubFigure {
  int width,height;

  LineSubFigure(int x, int y, int width, int height) {
    super(x,y);
    this.width=width;
    this.height=height;
  }

  public void draw(Graphics g) {
    g.drawLine(x,y,x+width,y+height);
  }
}


class BracketSubFigure implements Drawable {

  int[] leftBracketX, leftBracketY, rightBracketX, rightBracketY;

  BracketSubFigure(int x, int y, int width, int height,
		   int d, boolean angle) {
    int xr = x + width-1;
    int yu = y + height-1;
    if (angle) {
      int ym = y + height/2;
      leftBracketX=new int[] {x+d,x,x+d};
      leftBracketY=new int[] {y, ym, yu};
      rightBracketX=new int[] {xr-d, xr, xr-d};
      rightBracketY=new int[] {y, ym, yu};
    } else {
      leftBracketX=new int[] {x+d,x,x,x+d};
      leftBracketY=new int[] {y, y, yu, yu};
      rightBracketX=new int[] {xr-d, xr, xr, xr-d};
      rightBracketY=new int[] {y, y, yu, yu};
    }
  }

  public void draw(Graphics g) {
    g.drawPolyline(leftBracketX,leftBracketY,leftBracketX.length);
    g.drawPolyline(rightBracketX,rightBracketY,rightBracketX.length);
  }
}
