package de.renew.gui.xml;

import java.io.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import de.renew.gui.*;

public class XRNParser extends HandlerBase {
  public static CPNDrawing parse(InputStream stream)
    throws SAXException, IOException
  {
    InputSource inputSource=new InputSource(stream);
    Parser parser=null;
    try {
      try {
	parser=ParserFactory.makeParser();
      }
      catch (Exception e) {
	parser=ParserFactory.makeParser("com.ibm.xml.parsers.SAXParser");
      }
    } 
    catch (Exception e) {
      throw new RuntimeException("No SAX parser found.");
    }
    
    XRNParser handler=new XRNParser();
    parser.setDocumentHandler(handler);
    parser.setEntityResolver(handler);
    parser.parse(inputSource);
    return handler.result;
  }

  XRNParser() {
    result=new CPNDrawing();
    state=null;
  }

  private CPNDrawing result;
  private ParseState state;

  public InputSource resolveEntity (String publicId,String systemId) {
    if (systemId.startsWith("http://www.renew.de/")) {
      String dtd=systemId.substring(20);
      InputStream stream=getClass().getResourceAsStream(dtd);
      if (stream==null) {
	return null;
      } else {
	return new InputSource(stream);
      }
    } else {
      return null;
    }
  }

  public void startElement (String name, AttributeList atts)
    throws SAXException
  {
    if (state!=null) {
      state=state.startElement(name,atts);
    } else if ("net".equals(name)) {
      state=new NetParseState(result,atts);
    } else {
      state=new IgnoreElementState(null);
    }
  }
  public void endElement (String name)
    throws SAXException
  {
    if (state!=null) 
      state=state.endElement(name);
  }
  public void characters (char ch[], int start, int length)
    throws SAXException
  {
    if (state!=null) 
      state=state.characters(ch,start,length);
  }
}

