package de.renew.remote;

import java.rmi.*;

/**
 * Accessors allow to view and modify the state of simulation
 * related objects such as nets, places, transitions and their
 * instances.
 * <p>
 * Accessors are specified by remote interfaces that allow an
 * implementation that can be used via RMI.
 * </p>
 *
 * Accessor.java
 * Created: Mon Jul 17  2000
 * @author Michael Duvigneau
 *
 * @see NetAccessor
 * @see NetInstanceAccessor
 * @see PlaceAccessor
 * @see PlaceInstanceAccessor
 * @see TransitionAccessor
 * @see TransitionInstanceAccessor
 * @see ObjectAccessor
 */
public interface Accessor extends Remote {

  /**
   * Return the string representation of the accessed object as
   * generated by its <code>toString()</code> method.
   *
   * @return the object's string representation
   * @exception java.rmi.RemoteException if an RMI failure occured.
   */
  public String asString() throws RemoteException;
  
}
