package de.renew.shadow;

/**
 * Objects of this type act as a shadow compiler, but they
 * ignore the net system that has to be compiled and
 * return an empty shadow lookup.
 *
 * The idea is that we can use a null shadow compiler to
 * avoid any syntax checking without having to introduce special cases
 * in the GUI code.
 */
public class NullCompiler implements ShadowCompiler {
  public ShadowLookup compile(ShadowNetSystem netSystem) {
    return new ShadowLookup();
  }

  public String[] getLintNames() { return new String[0]; }

  public void lint(int lintNr, ShadowNetSystem netSystem,
    ShadowNet selectedNet, ShadowNet lastSelectedNet) {}
  public String checkDeclarationNode(String inscr, boolean special) {
    return null;
  }
  public String checkArcInscription(String inscr, boolean special,
				    ShadowNet shadowNet) {
    return null;
  }
  public String checkTransitionInscription(String inscr, boolean special,
					   ShadowNet shadowNet) {
    return null;
  }
  public String checkPlaceInscription(String inscr, boolean special,
				      ShadowNet shadowNet) {
    return null;
  }
}
