package de.renew.simulator;

import java.util.Enumeration;
import collections.*;
import de.renew.unify.*;

public class ChannelBinder implements Binder {
  // Try bindings with all the transitions possible
  // for the given net. Usually there will be very few
  // possible transitions.
  private Variable targetVariable;
  private String name;
  private Variable params;
  private Variable downlink;

  // If true, indicates that this channel need not be used if
  // the called net has no appropriate channels at all.
  private boolean isOptional;

  public ChannelBinder(Variable targetVariable, String name,
    Variable params, boolean isOptional)
  {
    this.targetVariable=targetVariable;
    this.name=name;
    this.params=params;
    this.isOptional=isOptional;
  }

  public int bindingBadness(Searcher searcher) {
    Object value=targetVariable.getValue();
    if (value instanceof ChannelTarget) {
      ChannelTarget channelTarget=(ChannelTarget)value;
      CollectionEnumeration uplinkProviders=
	channelTarget.getUplinkProviders(name);
      return BindingBadness.clip(uplinkProviders.numberOfRemainingElements());
    } else {
      return BindingBadness.max;
    }
  }

  public void bind(Searcher searcher) {
    ChannelTarget channelTarget=
      (ChannelTarget)targetVariable.getValue();
    CollectionEnumeration uplinkProviders=
      new RandomEnumeration(channelTarget.getUplinkProviders(name));

    if (isOptional && !uplinkProviders.hasMoreElements()) {
      // This channel is optional and no channel is offered at
      // the invoked net. This is too easy.
      searcher.search();
    } else {
      // Either the channel is not optional, or a suitable
      // channel is provided.
      while (uplinkProviders.hasMoreElements() && !searcher.isCompleted()) {
	UplinkProvider uplinkProvider=(UplinkProvider)
	  uplinkProviders.nextElement();
	uplinkProvider.bindChannel(params,searcher);
      }
    }
  }
}

