package de.renew.simulator;

import de.renew.expression.*;
import collections.*;
import de.renew.util.SingletonEnumeration;
import de.renew.util.ReflectionSerializer;
import java.io.IOException;

public class ClearArc implements TransitionInscription {
  Place place;
  Expression expression;
  boolean trace;

  /** 
   * This field is not really transient, but as <code>java.lang.Class
   * </code>is not always serializable, we have to store it by
   * ourselves.
   **/
  transient Class elementType;

  public ClearArc(Place place, Expression expression,
    Class elementType)
  {
    this.place=place;
    this.expression=expression;
    this.elementType=elementType;
    trace=true;
  }

  public void setTrace(boolean trace) {
    this.trace=trace;
  }

  public boolean getTrace() {
    return trace;
  }

  public CollectionEnumeration makeOccurrences(VariableMapper mapper,
    NetInstance netInstance, Searcher searcher)
  {
    return new SingletonEnumeration(new ClearArcOccurrence(this,
      mapper,netInstance));
  }

  /**
   * Serialization method, behaves like default writeObject
   * method. Stores the not-really-transient elementType field.
   **/
  private void writeObject(java.io.ObjectOutputStream out)
      throws IOException {
      out.defaultWriteObject();
      ReflectionSerializer.writeClass(out, elementType);
  }

  /**
   * Deserialization method, behaves like default readObject
   * method. Restores the not-really-transient elementType field.
   **/
  private void readObject(java.io.ObjectInputStream in)
      throws IOException, ClassNotFoundException {
      in.defaultReadObject();
      elementType = ReflectionSerializer.readClass(in);
  }

}

