package de.renew.simulator;

import collections.*;
import de.renew.unify.*;
import java.util.Vector;
import java.util.Enumeration;
import java.lang.reflect.Array;
import de.renew.function.Executor;
import de.renew.util.Value;

class FlexibleOutArcExecutable implements LateExecutable {
  PlaceInstance placeInstance;
  Variable tokenVar;
  FlexibleArc arc;

  FlexibleOutArcExecutable(PlaceInstance placeInstance, Variable tokenVar,
    FlexibleArc arc)
  {
    this.placeInstance=placeInstance;
    this.tokenVar=tokenVar;
    this.arc=arc;
  }

  public int phase() {
    return OUTPUT;
  }

  // We can put a token into an output place quickly.
  public boolean isLong() {
    return false;
  }

  private void putSingleToken(Tracer tracer, Object tok) {
    if (arc.forwardFunction!=null) {
      try {
	tok=arc.forwardFunction.function(tok);
      }
      catch (Impossible e) {
	// This should not happen.
      }
    }

    // We ignore the backward funtion. It is supposed to detect
    // errors for input arcs, but for output arcs it should not be set.

    if (arc.getTrace()) {
      tracer.trace("    Putting "+tok+
	" into "+placeInstance);
    }
    
    placeInstance.insertToken(tok,SearchQueue.getTime());  
  }

  public void execute(Tracer tracer) {
    Object tokens=tokenVar.getValue();

    placeInstance.lock.lock();
    if (tokens==null) {
      // Let's ignore this arc. Probably it we are not meant to
      // insert a single null token into the place, but rather 
      // the null is a substitute for an empty array.
      //
      // Actually, who knows? But we cannot complain, because we
      // are in the late phase of firing.
    } else if (tokens instanceof Enumeration) {
      // The place must be untyped, or a type error would have
      // occurred earlier on.
      Enumeration enum=(Enumeration)tokens;
      while (enum.hasMoreElements()) {
	putSingleToken(tracer,enum.nextElement());
      }
    } else if (tokens.getClass().isArray()) {
      int n=Array.getLength(tokens);

      Class elementType=tokens.getClass().getComponentType();
      for (int i=0;i<n;i++) {
	Object tok=Array.get(tokens,i);
	if (elementType.isPrimitive()) {
	  tok=new Value(tok);
	}
	putSingleToken(tracer,tok);
      }
    } else if (tokens instanceof List) {
      // The place must be untyped, or a type error would have
      // occurred earlier on.
      List current=(List)tokens;
      while (!current.isNull()) {
	putSingleToken(tracer,current.head());
	if (!(current.tail() instanceof List))  // stop if open/corrupted list
	  break; 
	current=(List)current.tail();
      }
    } else if (tokens instanceof Vector) {
      // The place must be untyped, or a type error would have
      // occurred earlier on.
      Vector vector=(Vector)tokens;
      for (int i=0;i<vector.size();i++) {
	putSingleToken(tracer,vector.elementAt(i));
      }
    } else {
      // As a last resort, we insert the value itself into the
      // place. This is barely acceptable, but at least we
      // know that the place is not typed, otherwise a type
      // check would have detected the illegal situation.
      putSingleToken(tracer,tokens);
    }

    placeInstance.lock.unlock();
  }
}
