package de.renew.simulator;

import collections.*;
import de.renew.unify.*;
import de.renew.util.Value;

public class InputArcBinder extends ArcRemoveBinder {
  Variable delayVar;

  protected InputArcBinder(Variable variable, Variable delayVar,
		 PlaceInstance placeInstance,
		 TokenReserver reserver)
  {
    super(variable,placeInstance,reserver);
    this.delayVar=delayVar;
  }

  private double getDelay() {
    Object timeObj=delayVar.getValue();
    if (timeObj instanceof Value) {
      timeObj=((Value)timeObj).value;
    }
    if (timeObj instanceof Number) {
      // For input arcs a positive delay forces earlier tokens.
      return ((Number)timeObj).doubleValue();
    } else {
      // Sorry, no such token.
      return Double.POSITIVE_INFINITY;
    }
  }

  protected boolean mayBind() {
    return Unify.isBound(delayVar);
  }

  protected boolean possible(Object token) {
    double delay=getDelay();
    return delay!=Double.POSITIVE_INFINITY &&
      reserver.containsRemovableToken(placeInstance,token,delay);
  }

  protected boolean remove(Object token) {
    double delay=getDelay();
    return delay!=Double.POSITIVE_INFINITY &&
      reserver.removeToken(placeInstance,token,delay);
  }

  protected void unremove(Object token) {
    reserver.unremoveToken(placeInstance,token,getDelay());
  }
}
