package de.renew.simulator;

import collections.CollectionEnumeration;
import java.util.NoSuchElementException;

/**
 * I am a special enumeration. I take another enumeration and
 * permute its elements in a random order.
 * 
 * @author Olaf Kummer
 **/

public class RandomEnumeration implements CollectionEnumeration {
  private Object[] data;
  private int size;

  public RandomEnumeration(CollectionEnumeration enum) {
    size=enum.numberOfRemainingElements();
    // Create array.
    data=new Object[size];
    // Copy all elements from the collection.
    for (int i=0;i<size;i++) {
      data[i]=enum.nextElement();
    }
  }

  public boolean hasMoreElements() {
    return size>0;
  }
  public Object nextElement() throws NoSuchElementException {
    if (size==0) {
      throw new NoSuchElementException();
    }

    // Choose a random element.
    int i=(int)(Math.random()*size);
    // Guard against rounding errors.
    if (i>=size) { i=size-1; }
    // Remember element.
    Object result=data[i];
    // Pull in last element.
    data[i]=data[size-1];
    // Null the field to allow garbage collection.
    data[size-1]=null;
    size=size-1;

    return result;
  }
  public boolean corrupted() {
    return false;
  }
  public int numberOfRemainingElements() {
    return size;
  }
}

