package de.renew.simulator;

import collections.*;
import java.io.Serializable;

public class TriggerCollection implements Serializable {
  // Keep track of triggering objects.
  private UpdatableSet triggers;
  private Triggerable triggerable;

  public TriggerCollection(Triggerable triggerable) {
    triggers=new HashedSet();
    this.triggerable=triggerable;
  }

  // The following two method must only be accessed through
  // the include and exclude methods of triggerable collections,
  // otherwise deadlocks might occur. Those method will make sure 
  // to lock this object.
  public void include(TriggerableCollection trigger) {
    triggers.include(trigger);
  }

  public void exclude(TriggerableCollection trigger) {
    triggers.exclude(trigger);
  }

  public void clear() {
    LinkedList list=new LinkedList();
    synchronized(this) {
      // Get the triggers.
      list.prependElements(triggers.elements());
    }

    // Send the notifications. No need to synchronize here.
    // Better allow concurrent accesses to reduce deadlock potential.
    // The triggerable collections are required to do the locking.
    CollectionEnumeration enum=list.elements();
    while (enum.hasMoreElements()) {
      TriggerableCollection triggerables=
	(TriggerableCollection)enum.nextElement();
      triggerables.exclude(triggerable);
    }
  }
}
