package de.uni_hamburg.fs;

import collections.*;
import java.util.*;
import de.renew.util.*;

public abstract class JavaClassType implements JavaType, ParsedType  {

  private Class type;
  private boolean isPrimitive;


  protected JavaClassType() { }

  protected JavaClassType(Class type) {
    setJavaClass(type);
  }

  void setJavaClass(Class type) {
    isPrimitive=type.isPrimitive();
    if (isPrimitive)
      this.type=Types.objectify(type);
    else
      this.type=type;
  }

  public Class getJavaClass() {
    if (isPrimitive)
	return Types.typify(type);
    else
	return type;
  }

  /** Return whether the feature <feature> is appropriate in this Type. */
  public boolean isApprop(Name featureName) {
      return false;
  }

  /** Return the required Type of the Value found under the given feature.
    * The feature has to be appropriate for this Type.
    */
  public Type appropType(Name featureName) throws NoSuchFeatureException {
      throw new NoSuchFeatureException(featureName,this);
  }

  /** Return an Enumeration of all appropriate features. */
  public CollectionEnumeration appropFeatureNames() {
      return EmptyEnumeration.INSTANCE;
  }

  /** Return a new node from this type. 
   */
  public Node newNode() {
      return new NoFeatureNode(this);
  }

  public String toString() {
      return getName();
  }

  public boolean equals(Object that) {
      if (that instanceof JavaClassType) {
	  return ((JavaClassType)that).getJavaClass()==getJavaClass();
      }
      return false;
  }

  public ParsedType unite(ParsedType that) throws UnificationFailure {
      if (that.equals(ParsedType.PARSED_TOP))
	  return this;
      if (that instanceof BasicType) {
	  return (BasicType)unify((BasicType)that);
      }
      throw new UnificationFailure();
  }

  public Type asType() throws UnificationFailure {
      return this;
  }

}
